# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import double
from .validators.logs import policytypes  # noqa: F401
from .validators.logs import (
    validate_loggroup_retention_in_days,
    validate_resource_policy,
)


class AccountPolicy(AWSObject):
    """
    `AccountPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-accountpolicy.html>`__
    """

    resource_type = "AWS::Logs::AccountPolicy"

    props: PropsDictType = {
        "PolicyDocument": (str, True),
        "PolicyName": (str, True),
        "PolicyType": (str, True),
        "Scope": (str, False),
        "SelectionCriteria": (str, False),
    }


class Delivery(AWSObject):
    """
    `Delivery <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-delivery.html>`__
    """

    resource_type = "AWS::Logs::Delivery"

    props: PropsDictType = {
        "DeliveryDestinationArn": (str, True),
        "DeliverySourceName": (str, True),
        "Tags": (Tags, False),
    }


class DeliveryDestination(AWSObject):
    """
    `DeliveryDestination <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-deliverydestination.html>`__
    """

    resource_type = "AWS::Logs::DeliveryDestination"

    props: PropsDictType = {
        "DeliveryDestinationPolicy": (dict, False),
        "DestinationResourceArn": (str, False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }


class DeliverySource(AWSObject):
    """
    `DeliverySource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-deliverysource.html>`__
    """

    resource_type = "AWS::Logs::DeliverySource"

    props: PropsDictType = {
        "LogType": (str, False),
        "Name": (str, True),
        "ResourceArn": (str, False),
        "Tags": (Tags, False),
    }


class Destination(AWSObject):
    """
    `Destination <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-destination.html>`__
    """

    resource_type = "AWS::Logs::Destination"

    props: PropsDictType = {
        "DestinationName": (str, True),
        "DestinationPolicy": (str, False),
        "RoleArn": (str, True),
        "TargetArn": (str, True),
    }


class LogAnomalyDetector(AWSObject):
    """
    `LogAnomalyDetector <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-loganomalydetector.html>`__
    """

    resource_type = "AWS::Logs::LogAnomalyDetector"

    props: PropsDictType = {
        "AccountId": (str, False),
        "AnomalyVisibilityTime": (double, False),
        "DetectorName": (str, False),
        "EvaluationFrequency": (str, False),
        "FilterPattern": (str, False),
        "KmsKeyId": (str, False),
        "LogGroupArnList": ([str], False),
    }


class LogGroup(AWSObject):
    """
    `LogGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-loggroup.html>`__
    """

    resource_type = "AWS::Logs::LogGroup"

    props: PropsDictType = {
        "DataProtectionPolicy": (dict, False),
        "KmsKeyId": (str, False),
        "LogGroupClass": (str, False),
        "LogGroupName": (str, False),
        "RetentionInDays": (validate_loggroup_retention_in_days, False),
        "Tags": (Tags, False),
    }


class LogStream(AWSObject):
    """
    `LogStream <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-logstream.html>`__
    """

    resource_type = "AWS::Logs::LogStream"

    props: PropsDictType = {
        "LogGroupName": (str, True),
        "LogStreamName": (str, False),
    }


class Dimension(AWSProperty):
    """
    `Dimension <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-logs-metricfilter-dimension.html>`__
    """

    props: PropsDictType = {
        "Key": (str, True),
        "Value": (str, True),
    }


class MetricTransformation(AWSProperty):
    """
    `MetricTransformation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-logs-metricfilter-metrictransformation.html>`__
    """

    props: PropsDictType = {
        "DefaultValue": (double, False),
        "Dimensions": ([Dimension], False),
        "MetricName": (str, True),
        "MetricNamespace": (str, True),
        "MetricValue": (str, True),
        "Unit": (str, False),
    }


class MetricFilter(AWSObject):
    """
    `MetricFilter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-metricfilter.html>`__
    """

    resource_type = "AWS::Logs::MetricFilter"

    props: PropsDictType = {
        "FilterName": (str, False),
        "FilterPattern": (str, True),
        "LogGroupName": (str, True),
        "MetricTransformations": ([MetricTransformation], True),
    }


class QueryDefinition(AWSObject):
    """
    `QueryDefinition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-querydefinition.html>`__
    """

    resource_type = "AWS::Logs::QueryDefinition"

    props: PropsDictType = {
        "LogGroupNames": ([str], False),
        "Name": (str, True),
        "QueryString": (str, True),
    }


class ResourcePolicy(AWSObject):
    """
    `ResourcePolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-resourcepolicy.html>`__
    """

    resource_type = "AWS::Logs::ResourcePolicy"

    props: PropsDictType = {
        "PolicyDocument": (validate_resource_policy, True),
        "PolicyName": (str, True),
    }


class SubscriptionFilter(AWSObject):
    """
    `SubscriptionFilter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-subscriptionfilter.html>`__
    """

    resource_type = "AWS::Logs::SubscriptionFilter"

    props: PropsDictType = {
        "DestinationArn": (str, True),
        "Distribution": (str, False),
        "FilterName": (str, False),
        "FilterPattern": (str, True),
        "LogGroupName": (str, True),
        "RoleArn": (str, False),
    }



# Extend the official tutorial to be more close to production, the original tutorial is located at:
# https://docs.aws.amazon.com/batch/latest/userguide/batch_sns_tutorial.html
# This example would send a successful job every 1 minute and failed job every 3 minutes.
# The failed jobs would invoke the lambda function through SNS.

from awacs.aws import Allow, PolicyDocument, Principal, Statement
from awacs.batch import SubmitJob
from awacs.sns import Publish

from troposphere import GetAtt, Join, Output, Parameter, Ref, Template
from troposphere.awslambda import Code, Function, Permission
from troposphere.batch import (
    ComputeEnvironment,
    ComputeEnvironmentOrder,
    ComputeResources,
    ContainerProperties,
    FargatePlatformConfiguration,
    JobDefinition,
    JobQueue,
    LogConfiguration,
    NetworkConfiguration,
    ResourceRequirement,
    RetryStrategy,
    Timeout,
)
from troposphere.events import BatchParameters, Rule, Target
from troposphere.iam import Policy, Role
from troposphere.sns import Subscription, Topic, TopicPolicy

t = Template(
    Description="Setup cron jobs with a job queue and send message through sns when a job fails."
)
t.set_version("2010-09-09")
t.set_transform("AWS::Serverless-2016-10-31")

BatchSG = t.add_parameter(
    Parameter(
        "BatchSG",
        Type="String",
    )
)

SubnetList = [
    t.add_parameter(
        Parameter(
            f"Subnet{i}",
            Type="String",
        )
    )
    for i in range(1, 4)
]


EventBridgeInvokeBatchJobQueueRole = t.add_resource(
    Role(
        "EventBridgeInvokeBatchJobQueueRole",
        AssumeRolePolicyDocument={
            "Statement": [
                {
                    "Effect": "Allow",
                    "Principal": {"Service": ["events.amazonaws.com"]},
                    "Action": ["sts:AssumeRole"],
                }
            ]
        },
        Policies=[
            Policy(
                PolicyDocument=PolicyDocument(
                    Version="2012-10-17",
                    Statement=[
                        Statement(
                            Effect=Allow,
                            Action=[SubmitJob],
                            Resource=["*"],
                        ),
                    ],
                ),
                PolicyName="EventBridgeInvokeBatchJobQueuePolicy",
            )
        ],
    )
)

BatchTaskFargateRole = t.add_resource(
    Role(
        "BatchTaskFargateRole",
        ManagedPolicyArns=[
            "arn:aws:iam::aws:policy/CloudWatchLogsFullAccess",
        ],
        AssumeRolePolicyDocument={
            "Statement": [
                {
                    "Effect": "Allow",
                    "Principal": {"Service": ["ecs-tasks.amazonaws.com"]},
                    "Action": ["sts:AssumeRole"],
                }
            ]
        },
    )
)


BatchComputeEnvironment = t.add_resource(
    ComputeEnvironment(
        "ComputeEnvironment",
        Type="MANAGED",
        ComputeEnvironmentName="cron-compute",
        ComputeResources=ComputeResources(
            MaxvCpus=16,
            SecurityGroupIds=[Ref(BatchSG)],
            Subnets=[Ref(SubNet) for SubNet in SubnetList],
            Type="FARGATE",
        ),
        ServiceRole=Join(
            ":",
            [
                "arn:aws:iam:",
                Ref("AWS::AccountId"),
                "role/aws-service-role/batch.amazonaws.com/AWSServiceRoleForBatch",
            ],
        ),
        State="ENABLED",
    )
)

JobQueueCron = t.add_resource(
    JobQueue(
        "JobQueueCron",
        JobQueueName="cron-queue",
        ComputeEnvironmentOrder=[
            ComputeEnvironmentOrder(
                ComputeEnvironment=Ref(BatchComputeEnvironment), Order=1
            ),
        ],
        Priority=60,
        State="ENABLED",
    )
)

# Make a failed job every 3 minutes, which should trigger the alert sns
JobDefinitionFailCron = t.add_resource(
    JobDefinition(
        "JobDefinitionFailCron",
        Type="container",
        ContainerProperties=ContainerProperties(
            Command=["/bin/sh", "-c", "exit 1"],
            Image="public.ecr.aws/amazonlinux/amazonlinux:latest",
            ExecutionRoleArn=GetAtt(BatchTaskFargateRole, "Arn"),
            FargatePlatformConfiguration=FargatePlatformConfiguration(
                PlatformVersion="1.4.0"
            ),
            NetworkConfiguration=NetworkConfiguration(AssignPublicIp="ENABLED"),
            ResourceRequirements=[
                ResourceRequirement(Type="MEMORY", Value="8192"),
                ResourceRequirement(Type="VCPU", Value="1"),
            ],
            JobRoleArn=GetAtt(BatchTaskFargateRole, "Arn"),
            LogConfiguration=LogConfiguration(LogDriver="awslogs"),
        ),
        RetryStrategy=RetryStrategy(Attempts=3),
        Timeout=Timeout(AttemptDurationSeconds=3600),
        PlatformCapabilities=["FARGATE"],
        JobDefinitionName="cron-fail",
    )
)

JobDefinitionSuccessCron = t.add_resource(
    JobDefinition(
        "JobDefinitionSuccessCron",
        Type="container",
        ContainerProperties=ContainerProperties(
            Command=["echo", "'Success'"],
            Image="public.ecr.aws/amazonlinux/amazonlinux:latest",
            ExecutionRoleArn=GetAtt(BatchTaskFargateRole, "Arn"),
            FargatePlatformConfiguration=FargatePlatformConfiguration(
                PlatformVersion="1.4.0"
            ),
            NetworkConfiguration=NetworkConfiguration(AssignPublicIp="ENABLED"),
            ResourceRequirements=[
                ResourceRequirement(Type="MEMORY", Value="8192"),
                ResourceRequirement(Type="VCPU", Value="1"),
            ],
            JobRoleArn=GetAtt(BatchTaskFargateRole, "Arn"),
            LogConfiguration=LogConfiguration(LogDriver="awslogs"),
        ),
        RetryStrategy=RetryStrategy(Attempts=3),
        Timeout=Timeout(AttemptDurationSeconds=3600),
        PlatformCapabilities=["FARGATE"],
        JobDefinitionName="cron-success",
    )
)


JobFailRevision = 1
EventRuleCronFailJob = t.add_resource(
    Rule(
        "EventRuleCronFailJob",
        Name="cron-fail",
        ScheduleExpression="cron(*/3 * ? * * *)",
        State="ENABLED",
        Targets=[
            Target(
                "JobQueueFailCronAsTarget",
                Arn=Ref(JobQueueCron),
                RoleArn=GetAtt(EventBridgeInvokeBatchJobQueueRole, "Arn"),
                BatchParameters=BatchParameters(
                    JobDefinition=Join(
                        ":",
                        [
                            "arn:aws:batch",
                            Ref("AWS::Region"),
                            Ref("AWS::AccountId"),
                            "job-definition/cron-fail",
                            JobFailRevision,
                        ],
                    ),
                    JobName="cron-fail-eventbridge",
                ),
                Id="cron-fail",
            ),
        ],
    )
)

JobSuccessRevision = 1
EventRuleCronSuccessJob = t.add_resource(
    Rule(
        "EventRuleCronSuccessJob",
        Name="cron-success",
        ScheduleExpression="cron(*/1 * ? * * *)",
        State="ENABLED",
        Targets=[
            Target(
                "JobQueueSuccessCronAsTarget",
                Arn=Ref(JobQueueCron),
                RoleArn=GetAtt(EventBridgeInvokeBatchJobQueueRole, "Arn"),
                BatchParameters=BatchParameters(
                    JobDefinition=Join(
                        ":",
                        [
                            "arn:aws:batch",
                            Ref("AWS::Region"),
                            Ref("AWS::AccountId"),
                            "job-definition/cron-success",
                            JobSuccessRevision,
                        ],
                    ),
                    JobName="cron-success-eventbridge",
                ),
                Id="cron-success",
            ),
        ],
    )
)


LambdaExecutionRole = t.add_resource(
    Role(
        "LambdaExecutionRole",
        Path="/",
        Policies=[
            Policy(
                PolicyName="root",
                PolicyDocument={
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Action": ["logs:*"],
                            "Resource": "arn:aws:logs:*:*:*",
                            "Effect": "Allow",
                        },
                        {"Action": ["lambda:*"], "Resource": "*", "Effect": "Allow"},
                    ],
                },
            )
        ],
        AssumeRolePolicyDocument={
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Action": ["sts:AssumeRole"],
                    "Effect": "Allow",
                    "Principal": {"Service": ["lambda.amazonaws.com"]},
                }
            ],
        },
    )
)


LambdaCronFailNotify = t.add_resource(
    Function(
        "LambdaCronFailNotify",
        Handler="index.lambda_handler",
        Code=Code(
            ZipFile="""
import boto3
import json


def lambda_handler(event, context):
    print(event)  # implement notification code here, ex; to slack, telegram, etc.
    return {
        "statusCode": 200,
        "body": json.dumps({
            "message": "ok",
        })
    }
"""
        ),
        FunctionName="lambda_cron_fail_notify",
        Role=GetAtt(LambdaExecutionRole, "Arn"),
        Runtime="python3.9",
        MemorySize=128,
        Timeout=900,
    )
)


SnsTopicCronFailed = t.add_resource(
    Topic(
        "SnsTopicCronFailed",
        Subscription=[
            Subscription(
                Endpoint=GetAtt(LambdaCronFailNotify, "Arn"),
                Protocol="lambda",
            )
        ],
        DisplayName="SnsTopicCronFailed",
        TopicName="SnsTopicCronFailed",
    )
)


t.add_resource(
    TopicPolicy(
        "CronFailedTopicPolicy",
        PolicyDocument=PolicyDocument(
            Statement=[
                Statement(
                    Effect=Allow,
                    Action=[Publish],
                    Principal=Principal("Service", ["events.amazonaws.com"]),
                    Resource=[Ref(SnsTopicCronFailed)],
                ),
            ],
        ),
        Topics=[Ref(SnsTopicCronFailed)],
    )
)


EventRuleCronFailed = t.add_resource(
    Rule(
        "EventRuleCronFailed",
        Name="cron-failed",
        Description="Send SNS for cron jobs of cron-queue for which status are FAILED",
        State="ENABLED",
        EventPattern={
            "detail-type": ["Batch Job State Change"],
            "source": ["aws.batch"],
            "detail": {"status": ["FAILED"], "jobQueue": [Ref(JobQueueCron)]},
        },
        Targets=[
            Target(
                "CronFailedAsTarget",
                Arn=Ref(SnsTopicCronFailed),
                Id="cron-failed",
            ),
        ],
    )
)

LambdaInvokePermissionCronFailed = t.add_resource(
    Permission(
        "LambdaInvokePermissionCronFailed",
        Action="lambda:InvokeFunction",
        FunctionName=Ref(LambdaCronFailNotify),
        Principal="sns.amazonaws.com",
        SourceArn=Ref(SnsTopicCronFailed),
    )
)

t.add_output(
    [
        Output("BatchComputeEnvironment", Value=Ref(BatchComputeEnvironment)),
        Output("JobQueueCron", Value=Ref(JobQueueCron)),
        Output("EventRuleCronFailJob", Value=Ref(EventRuleCronFailJob)),
        Output("EventRuleCronSuccessJob", Value=Ref(EventRuleCronSuccessJob)),
        Output("SnsTopicCronFailed", Value=Ref(SnsTopicCronFailed)),
        Output("LambdaCronFailNotify", Value=Ref(LambdaCronFailNotify)),
    ]
)


print(t.to_json())



import awacs.ecr as ecr
import awacs.iam as iam
from awacs.aws import Allow, AWSPrincipal, PolicyDocument, Statement

from troposphere import Template
from troposphere.ecr import Repository

t = Template()

t.add_resource(
    Repository(
        "MyRepository",
        RepositoryName="test-repository",
        RepositoryPolicyText=PolicyDocument(
            Version="2008-10-17",
            Statement=[
                Statement(
                    Sid="AllowPushPull",
                    Effect=Allow,
                    Principal=AWSPrincipal(
                        [
                            iam.ARN(account="123456789012", resource="user/Bob"),
                            iam.ARN(account="123456789012", resource="user/Alice"),
                        ]
                    ),
                    Action=[
                        ecr.GetDownloadUrlForLayer,
                        ecr.BatchGetImage,
                        ecr.BatchCheckLayerAvailability,
                        ecr.PutImage,
                        ecr.InitiateLayerUpload,
                        ecr.UploadLayerPart,
                        ecr.CompleteLayerUpload,
                    ],
                ),
            ],
        ),
    )
)

print(t.to_json())



from troposphere import Ref, Template
from troposphere.applicationautoscaling import (
    ScalableTarget,
    ScalingPolicy,
    StepAdjustment,
    StepScalingPolicyConfiguration,
)

t = Template()

scalable_target = ScalableTarget(
    "scalableTarget",
    MaxCapacity=2,
    MinCapacity=1,
    ResourceId="service/ecsStack",
    RoleARN="Access Management (IAM) role",
    ScalableDimension="ecs:service:DesiredCount",
    ServiceNamespace="ecs",
)

scaling_policy = ScalingPolicy(
    "scalingPolicy",
    PolicyName="AStepPolicy",
    PolicyType="StepScaling",
    ScalingTargetId=Ref(scalable_target),
    StepScalingPolicyConfiguration=StepScalingPolicyConfiguration(
        AdjustmentType="PercentChangeInCapacity",
        Cooldown=60,
        MetricAggregationType="Average",
        StepAdjustments=[
            StepAdjustment(
                MetricIntervalLowerBound=0,
                ScalingAdjustment=200,
            ),
        ],
    ),
)

t = Template()

t.add_resource(scalable_target)
t.add_resource(scaling_policy)

print(t.to_json())



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, double, integer
from .validators.iot import policytypes, validate_json_checker


class AuditCheckConfiguration(AWSProperty):
    """
    `AuditCheckConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-accountauditconfiguration-auditcheckconfiguration.html>`__
    """

    props: PropsDictType = {
        "Enabled": (boolean, False),
    }


class AuditCheckConfigurations(AWSProperty):
    """
    `AuditCheckConfigurations <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-accountauditconfiguration-auditcheckconfigurations.html>`__
    """

    props: PropsDictType = {
        "AuthenticatedCognitoRoleOverlyPermissiveCheck": (
            AuditCheckConfiguration,
            False,
        ),
        "CaCertificateExpiringCheck": (AuditCheckConfiguration, False),
        "CaCertificateKeyQualityCheck": (AuditCheckConfiguration, False),
        "ConflictingClientIdsCheck": (AuditCheckConfiguration, False),
        "DeviceCertificateExpiringCheck": (AuditCheckConfiguration, False),
        "DeviceCertificateKeyQualityCheck": (AuditCheckConfiguration, False),
        "DeviceCertificateSharedCheck": (AuditCheckConfiguration, False),
        "IntermediateCaRevokedForActiveDeviceCertificatesCheck": (
            AuditCheckConfiguration,
            False,
        ),
        "IoTPolicyPotentialMisConfigurationCheck": (AuditCheckConfiguration, False),
        "IotPolicyOverlyPermissiveCheck": (AuditCheckConfiguration, False),
        "IotRoleAliasAllowsAccessToUnusedServicesCheck": (
            AuditCheckConfiguration,
            False,
        ),
        "IotRoleAliasOverlyPermissiveCheck": (AuditCheckConfiguration, False),
        "LoggingDisabledCheck": (AuditCheckConfiguration, False),
        "RevokedCaCertificateStillActiveCheck": (AuditCheckConfiguration, False),
        "RevokedDeviceCertificateStillActiveCheck": (AuditCheckConfiguration, False),
        "UnauthenticatedCognitoRoleOverlyPermissiveCheck": (
            AuditCheckConfiguration,
            False,
        ),
    }


class AuditNotificationTarget(AWSProperty):
    """
    `AuditNotificationTarget <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-accountauditconfiguration-auditnotificationtarget.html>`__
    """

    props: PropsDictType = {
        "Enabled": (boolean, False),
        "RoleArn": (str, False),
        "TargetArn": (str, False),
    }


class AuditNotificationTargetConfigurations(AWSProperty):
    """
    `AuditNotificationTargetConfigurations <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-accountauditconfiguration-auditnotificationtargetconfigurations.html>`__
    """

    props: PropsDictType = {
        "Sns": (AuditNotificationTarget, False),
    }


class AccountAuditConfiguration(AWSObject):
    """
    `AccountAuditConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-accountauditconfiguration.html>`__
    """

    resource_type = "AWS::IoT::AccountAuditConfiguration"

    props: PropsDictType = {
        "AccountId": (str, True),
        "AuditCheckConfigurations": (AuditCheckConfigurations, True),
        "AuditNotificationTargetConfigurations": (
            AuditNotificationTargetConfigurations,
            False,
        ),
        "RoleArn": (str, True),
    }


class Authorizer(AWSObject):
    """
    `Authorizer <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-authorizer.html>`__
    """

    resource_type = "AWS::IoT::Authorizer"

    props: PropsDictType = {
        "AuthorizerFunctionArn": (str, True),
        "AuthorizerName": (str, False),
        "EnableCachingForHttp": (boolean, False),
        "SigningDisabled": (boolean, False),
        "Status": (str, False),
        "Tags": (Tags, False),
        "TokenKeyName": (str, False),
        "TokenSigningPublicKeys": (dict, False),
    }


class BillingGroupProperties(AWSProperty):
    """
    `BillingGroupProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-billinggroup-billinggroupproperties.html>`__
    """

    props: PropsDictType = {
        "BillingGroupDescription": (str, False),
    }


class BillingGroup(AWSObject):
    """
    `BillingGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-billinggroup.html>`__
    """

    resource_type = "AWS::IoT::BillingGroup"

    props: PropsDictType = {
        "BillingGroupName": (str, False),
        "BillingGroupProperties": (BillingGroupProperties, False),
        "Tags": (Tags, False),
    }


class RegistrationConfig(AWSProperty):
    """
    `RegistrationConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-cacertificate-registrationconfig.html>`__
    """

    props: PropsDictType = {
        "RoleArn": (str, False),
        "TemplateBody": (str, False),
        "TemplateName": (str, False),
    }


class CACertificate(AWSObject):
    """
    `CACertificate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-cacertificate.html>`__
    """

    resource_type = "AWS::IoT::CACertificate"

    props: PropsDictType = {
        "AutoRegistrationStatus": (str, False),
        "CACertificatePem": (str, True),
        "CertificateMode": (str, False),
        "RegistrationConfig": (RegistrationConfig, False),
        "RemoveAutoRegistration": (boolean, False),
        "Status": (str, True),
        "Tags": (Tags, False),
        "VerificationCertificatePem": (str, False),
    }


class Certificate(AWSObject):
    """
    `Certificate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-certificate.html>`__
    """

    resource_type = "AWS::IoT::Certificate"

    props: PropsDictType = {
        "CACertificatePem": (str, False),
        "CertificateMode": (str, False),
        "CertificatePem": (str, False),
        "CertificateSigningRequest": (str, False),
        "Status": (str, True),
    }


class CertificateProvider(AWSObject):
    """
    `CertificateProvider <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-certificateprovider.html>`__
    """

    resource_type = "AWS::IoT::CertificateProvider"

    props: PropsDictType = {
        "AccountDefaultForOperations": ([str], True),
        "CertificateProviderName": (str, False),
        "LambdaFunctionArn": (str, True),
        "Tags": (Tags, False),
    }


class CustomMetric(AWSObject):
    """
    `CustomMetric <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-custommetric.html>`__
    """

    resource_type = "AWS::IoT::CustomMetric"

    props: PropsDictType = {
        "DisplayName": (str, False),
        "MetricName": (str, False),
        "MetricType": (str, True),
        "Tags": (Tags, False),
    }


class Dimension(AWSObject):
    """
    `Dimension <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-dimension.html>`__
    """

    resource_type = "AWS::IoT::Dimension"

    props: PropsDictType = {
        "Name": (str, False),
        "StringValues": ([str], True),
        "Tags": (Tags, False),
        "Type": (str, True),
    }


class AuthorizerConfig(AWSProperty):
    """
    `AuthorizerConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-domainconfiguration-authorizerconfig.html>`__
    """

    props: PropsDictType = {
        "AllowAuthorizerOverride": (boolean, False),
        "DefaultAuthorizerName": (str, False),
    }


class TlsConfig(AWSProperty):
    """
    `TlsConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-domainconfiguration-tlsconfig.html>`__
    """

    props: PropsDictType = {
        "SecurityPolicy": (str, False),
    }


class DomainConfiguration(AWSObject):
    """
    `DomainConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-domainconfiguration.html>`__
    """

    resource_type = "AWS::IoT::DomainConfiguration"

    props: PropsDictType = {
        "AuthorizerConfig": (AuthorizerConfig, False),
        "DomainConfigurationName": (str, False),
        "DomainConfigurationStatus": (str, False),
        "DomainName": (str, False),
        "ServerCertificateArns": ([str], False),
        "ServiceType": (str, False),
        "Tags": (Tags, False),
        "TlsConfig": (TlsConfig, False),
        "ValidationCertificateArn": (str, False),
    }


class AggregationType(AWSProperty):
    """
    `AggregationType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-fleetmetric-aggregationtype.html>`__
    """

    props: PropsDictType = {
        "Name": (str, True),
        "Values": ([str], True),
    }


class FleetMetric(AWSObject):
    """
    `FleetMetric <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-fleetmetric.html>`__
    """

    resource_type = "AWS::IoT::FleetMetric"

    props: PropsDictType = {
        "AggregationField": (str, False),
        "AggregationType": (AggregationType, False),
        "Description": (str, False),
        "IndexName": (str, False),
        "MetricName": (str, True),
        "Period": (integer, False),
        "QueryString": (str, False),
        "QueryVersion": (str, False),
        "Tags": (Tags, False),
        "Unit": (str, False),
    }


class AbortCriteria(AWSProperty):
    """
    `AbortCriteria <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-jobtemplate-abortcriteria.html>`__
    """

    props: PropsDictType = {
        "Action": (str, True),
        "FailureType": (str, True),
        "MinNumberOfExecutedThings": (integer, True),
        "ThresholdPercentage": (double, True),
    }


class AbortConfig(AWSProperty):
    """
    `AbortConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-jobtemplate-abortconfig.html>`__
    """

    props: PropsDictType = {
        "CriteriaList": ([AbortCriteria], True),
    }


class RetryCriteria(AWSProperty):
    """
    `RetryCriteria <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-jobtemplate-retrycriteria.html>`__
    """

    props: PropsDictType = {
        "FailureType": (str, False),
        "NumberOfRetries": (integer, False),
    }


class JobExecutionsRetryConfig(AWSProperty):
    """
    `JobExecutionsRetryConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-jobtemplate-jobexecutionsretryconfig.html>`__
    """

    props: PropsDictType = {
        "RetryCriteriaList": ([RetryCriteria], False),
    }


class RateIncreaseCriteria(AWSProperty):
    """
    `RateIncreaseCriteria <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-jobtemplate-rateincreasecriteria.html>`__
    """

    props: PropsDictType = {
        "NumberOfNotifiedThings": (integer, False),
        "NumberOfSucceededThings": (integer, False),
    }


class ExponentialRolloutRate(AWSProperty):
    """
    `ExponentialRolloutRate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-jobtemplate-exponentialrolloutrate.html>`__
    """

    props: PropsDictType = {
        "BaseRatePerMinute": (integer, True),
        "IncrementFactor": (double, True),
        "RateIncreaseCriteria": (RateIncreaseCriteria, True),
    }


class JobExecutionsRolloutConfig(AWSProperty):
    """
    `JobExecutionsRolloutConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-jobtemplate-jobexecutionsrolloutconfig.html>`__
    """

    props: PropsDictType = {
        "ExponentialRolloutRate": (ExponentialRolloutRate, False),
        "MaximumPerMinute": (integer, False),
    }


class MaintenanceWindow(AWSProperty):
    """
    `MaintenanceWindow <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-jobtemplate-maintenancewindow.html>`__
    """

    props: PropsDictType = {
        "DurationInMinutes": (integer, False),
        "StartTime": (str, False),
    }


class PresignedUrlConfig(AWSProperty):
    """
    `PresignedUrlConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-jobtemplate-presignedurlconfig.html>`__
    """

    props: PropsDictType = {
        "ExpiresInSec": (integer, False),
        "RoleArn": (str, True),
    }


class TimeoutConfig(AWSProperty):
    """
    `TimeoutConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-jobtemplate-timeoutconfig.html>`__
    """

    props: PropsDictType = {
        "InProgressTimeoutInMinutes": (integer, True),
    }


class JobTemplate(AWSObject):
    """
    `JobTemplate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-jobtemplate.html>`__
    """

    resource_type = "AWS::IoT::JobTemplate"

    props: PropsDictType = {
        "AbortConfig": (validate_json_checker, False),
        "Description": (str, True),
        "DestinationPackageVersions": ([str], False),
        "Document": (str, False),
        "DocumentSource": (str, False),
        "JobArn": (str, False),
        "JobExecutionsRetryConfig": (JobExecutionsRetryConfig, False),
        "JobExecutionsRolloutConfig": (validate_json_checker, False),
        "JobTemplateId": (str, True),
        "MaintenanceWindows": ([MaintenanceWindow], False),
        "PresignedUrlConfig": (PresignedUrlConfig, False),
        "Tags": (Tags, False),
        "TimeoutConfig": (validate_json_checker, False),
    }


class Logging(AWSObject):
    """
    `Logging <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-logging.html>`__
    """

    resource_type = "AWS::IoT::Logging"

    props: PropsDictType = {
        "AccountId": (str, True),
        "DefaultLogLevel": (str, True),
        "RoleArn": (str, True),
    }


class AddThingsToThingGroupParams(AWSProperty):
    """
    `AddThingsToThingGroupParams <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-mitigationaction-addthingstothinggroupparams.html>`__
    """

    props: PropsDictType = {
        "OverrideDynamicGroups": (boolean, False),
        "ThingGroupNames": ([str], True),
    }


class EnableIoTLoggingParams(AWSProperty):
    """
    `EnableIoTLoggingParams <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-mitigationaction-enableiotloggingparams.html>`__
    """

    props: PropsDictType = {
        "LogLevel": (str, True),
        "RoleArnForLogging": (str, True),
    }


class PublishFindingToSnsParams(AWSProperty):
    """
    `PublishFindingToSnsParams <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-mitigationaction-publishfindingtosnsparams.html>`__
    """

    props: PropsDictType = {
        "TopicArn": (str, True),
    }


class ReplaceDefaultPolicyVersionParams(AWSProperty):
    """
    `ReplaceDefaultPolicyVersionParams <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-mitigationaction-replacedefaultpolicyversionparams.html>`__
    """

    props: PropsDictType = {
        "TemplateName": (str, True),
    }


class UpdateCACertificateParams(AWSProperty):
    """
    `UpdateCACertificateParams <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-mitigationaction-updatecacertificateparams.html>`__
    """

    props: PropsDictType = {
        "Action": (str, True),
    }


class UpdateDeviceCertificateParams(AWSProperty):
    """
    `UpdateDeviceCertificateParams <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-mitigationaction-updatedevicecertificateparams.html>`__
    """

    props: PropsDictType = {
        "Action": (str, True),
    }


class ActionParams(AWSProperty):
    """
    `ActionParams <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-mitigationaction-actionparams.html>`__
    """

    props: PropsDictType = {
        "AddThingsToThingGroupParams": (AddThingsToThingGroupParams, False),
        "EnableIoTLoggingParams": (EnableIoTLoggingParams, False),
        "PublishFindingToSnsParams": (PublishFindingToSnsParams, False),
        "ReplaceDefaultPolicyVersionParams": (ReplaceDefaultPolicyVersionParams, False),
        "UpdateCACertificateParams": (UpdateCACertificateParams, False),
        "UpdateDeviceCertificateParams": (UpdateDeviceCertificateParams, False),
    }


class MitigationAction(AWSObject):
    """
    `MitigationAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-mitigationaction.html>`__
    """

    resource_type = "AWS::IoT::MitigationAction"

    props: PropsDictType = {
        "ActionName": (str, False),
        "ActionParams": (ActionParams, True),
        "RoleArn": (str, True),
        "Tags": (Tags, False),
    }


class Policy(AWSObject):
    """
    `Policy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policy.html>`__
    """

    resource_type = "AWS::IoT::Policy"

    props: PropsDictType = {
        "PolicyDocument": (policytypes, True),
        "PolicyName": (str, False),
        "Tags": (Tags, False),
    }


class PolicyPrincipalAttachment(AWSObject):
    """
    `PolicyPrincipalAttachment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policyprincipalattachment.html>`__
    """

    resource_type = "AWS::IoT::PolicyPrincipalAttachment"

    props: PropsDictType = {
        "PolicyName": (str, True),
        "Principal": (str, True),
    }


class ProvisioningHook(AWSProperty):
    """
    `ProvisioningHook <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-provisioningtemplate-provisioninghook.html>`__
    """

    props: PropsDictType = {
        "PayloadVersion": (str, False),
        "TargetArn": (str, False),
    }


class ProvisioningTemplate(AWSObject):
    """
    `ProvisioningTemplate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-provisioningtemplate.html>`__
    """

    resource_type = "AWS::IoT::ProvisioningTemplate"

    props: PropsDictType = {
        "Description": (str, False),
        "Enabled": (boolean, False),
        "PreProvisioningHook": (ProvisioningHook, False),
        "ProvisioningRoleArn": (str, True),
        "Tags": (Tags, False),
        "TemplateBody": (str, True),
        "TemplateName": (str, False),
        "TemplateType": (str, False),
    }


class ResourceSpecificLogging(AWSObject):
    """
    `ResourceSpecificLogging <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-resourcespecificlogging.html>`__
    """

    resource_type = "AWS::IoT::ResourceSpecificLogging"

    props: PropsDictType = {
        "LogLevel": (str, True),
        "TargetName": (str, True),
        "TargetType": (str, True),
    }


class RoleAlias(AWSObject):
    """
    `RoleAlias <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-rolealias.html>`__
    """

    resource_type = "AWS::IoT::RoleAlias"

    props: PropsDictType = {
        "CredentialDurationSeconds": (integer, False),
        "RoleAlias": (str, False),
        "RoleArn": (str, True),
        "Tags": (Tags, False),
    }


class ScheduledAudit(AWSObject):
    """
    `ScheduledAudit <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-scheduledaudit.html>`__
    """

    resource_type = "AWS::IoT::ScheduledAudit"

    props: PropsDictType = {
        "DayOfMonth": (str, False),
        "DayOfWeek": (str, False),
        "Frequency": (str, True),
        "ScheduledAuditName": (str, False),
        "Tags": (Tags, False),
        "TargetCheckNames": ([str], True),
    }


class AlertTarget(AWSProperty):
    """
    `AlertTarget <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-securityprofile-alerttarget.html>`__
    """

    props: PropsDictType = {
        "AlertTargetArn": (str, True),
        "RoleArn": (str, True),
    }


class MachineLearningDetectionConfig(AWSProperty):
    """
    `MachineLearningDetectionConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-securityprofile-machinelearningdetectionconfig.html>`__
    """

    props: PropsDictType = {
        "ConfidenceLevel": (str, False),
    }


class MetricValue(AWSProperty):
    """
    `MetricValue <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-securityprofile-metricvalue.html>`__
    """

    props: PropsDictType = {
        "Cidrs": ([str], False),
        "Count": (str, False),
        "Number": (double, False),
        "Numbers": ([double], False),
        "Ports": ([integer], False),
        "Strings": ([str], False),
    }


class StatisticalThreshold(AWSProperty):
    """
    `StatisticalThreshold <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-securityprofile-statisticalthreshold.html>`__
    """

    props: PropsDictType = {
        "Statistic": (str, False),
    }


class BehaviorCriteria(AWSProperty):
    """
    `BehaviorCriteria <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-securityprofile-behaviorcriteria.html>`__
    """

    props: PropsDictType = {
        "ComparisonOperator": (str, False),
        "ConsecutiveDatapointsToAlarm": (integer, False),
        "ConsecutiveDatapointsToClear": (integer, False),
        "DurationSeconds": (integer, False),
        "MlDetectionConfig": (MachineLearningDetectionConfig, False),
        "StatisticalThreshold": (StatisticalThreshold, False),
        "Value": (MetricValue, False),
    }


class MetricDimension(AWSProperty):
    """
    `MetricDimension <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-securityprofile-metricdimension.html>`__
    """

    props: PropsDictType = {
        "DimensionName": (str, True),
        "Operator": (str, False),
    }


class Behavior(AWSProperty):
    """
    `Behavior <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-securityprofile-behavior.html>`__
    """

    props: PropsDictType = {
        "Criteria": (BehaviorCriteria, False),
        "ExportMetric": (boolean, False),
        "Metric": (str, False),
        "MetricDimension": (MetricDimension, False),
        "Name": (str, True),
        "SuppressAlerts": (boolean, False),
    }


class MetricToRetain(AWSProperty):
    """
    `MetricToRetain <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-securityprofile-metrictoretain.html>`__
    """

    props: PropsDictType = {
        "ExportMetric": (boolean, False),
        "Metric": (str, True),
        "MetricDimension": (MetricDimension, False),
    }


class MetricsExportConfig(AWSProperty):
    """
    `MetricsExportConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-securityprofile-metricsexportconfig.html>`__
    """

    props: PropsDictType = {
        "MqttTopic": (str, True),
        "RoleArn": (str, True),
    }


class SecurityProfile(AWSObject):
    """
    `SecurityProfile <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-securityprofile.html>`__
    """

    resource_type = "AWS::IoT::SecurityProfile"

    props: PropsDictType = {
        "AdditionalMetricsToRetainV2": ([MetricToRetain], False),
        "AlertTargets": (dict, False),
        "Behaviors": ([Behavior], False),
        "MetricsExportConfig": (MetricsExportConfig, False),
        "SecurityProfileDescription": (str, False),
        "SecurityProfileName": (str, False),
        "Tags": (Tags, False),
        "TargetArns": ([str], False),
    }


class SoftwarePackage(AWSObject):
    """
    `SoftwarePackage <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-softwarepackage.html>`__
    """

    resource_type = "AWS::IoT::SoftwarePackage"

    props: PropsDictType = {
        "Description": (str, False),
        "PackageName": (str, False),
        "Tags": (Tags, False),
    }


class SoftwarePackageVersion(AWSObject):
    """
    `SoftwarePackageVersion <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-softwarepackageversion.html>`__
    """

    resource_type = "AWS::IoT::SoftwarePackageVersion"

    props: PropsDictType = {
        "Attributes": (dict, False),
        "Description": (str, False),
        "PackageName": (str, True),
        "Tags": (Tags, False),
        "VersionName": (str, False),
    }


class AttributePayload(AWSProperty):
    """
    `AttributePayload <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-thinggroup-attributepayload.html>`__
    """

    props: PropsDictType = {
        "Attributes": (dict, False),
    }


class Thing(AWSObject):
    """
    `Thing <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thing.html>`__
    """

    resource_type = "AWS::IoT::Thing"

    props: PropsDictType = {
        "AttributePayload": (AttributePayload, False),
        "ThingName": (str, False),
    }


class ThingGroupProperties(AWSProperty):
    """
    `ThingGroupProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-thinggroup-thinggroupproperties.html>`__
    """

    props: PropsDictType = {
        "AttributePayload": (AttributePayload, False),
        "ThingGroupDescription": (str, False),
    }


class ThingGroup(AWSObject):
    """
    `ThingGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thinggroup.html>`__
    """

    resource_type = "AWS::IoT::ThingGroup"

    props: PropsDictType = {
        "ParentGroupName": (str, False),
        "QueryString": (str, False),
        "Tags": (Tags, False),
        "ThingGroupName": (str, False),
        "ThingGroupProperties": (ThingGroupProperties, False),
    }


class ThingPrincipalAttachment(AWSObject):
    """
    `ThingPrincipalAttachment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thingprincipalattachment.html>`__
    """

    resource_type = "AWS::IoT::ThingPrincipalAttachment"

    props: PropsDictType = {
        "Principal": (str, True),
        "ThingName": (str, True),
    }


class ThingTypeProperties(AWSProperty):
    """
    `ThingTypeProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-thingtype-thingtypeproperties.html>`__
    """

    props: PropsDictType = {
        "SearchableAttributes": ([str], False),
        "ThingTypeDescription": (str, False),
    }


class ThingType(AWSObject):
    """
    `ThingType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thingtype.html>`__
    """

    resource_type = "AWS::IoT::ThingType"

    props: PropsDictType = {
        "DeprecateThingType": (boolean, False),
        "Tags": (Tags, False),
        "ThingTypeName": (str, False),
        "ThingTypeProperties": (ThingTypeProperties, False),
    }


class CloudwatchAlarmAction(AWSProperty):
    """
    `CloudwatchAlarmAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchalarmaction.html>`__
    """

    props: PropsDictType = {
        "AlarmName": (str, True),
        "RoleArn": (str, True),
        "StateReason": (str, True),
        "StateValue": (str, True),
    }


class CloudwatchLogsAction(AWSProperty):
    """
    `CloudwatchLogsAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchlogsaction.html>`__
    """

    props: PropsDictType = {
        "BatchMode": (boolean, False),
        "LogGroupName": (str, True),
        "RoleArn": (str, True),
    }


class CloudwatchMetricAction(AWSProperty):
    """
    `CloudwatchMetricAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchmetricaction.html>`__
    """

    props: PropsDictType = {
        "MetricName": (str, True),
        "MetricNamespace": (str, True),
        "MetricTimestamp": (str, False),
        "MetricUnit": (str, True),
        "MetricValue": (str, True),
        "RoleArn": (str, True),
    }


class DynamoDBAction(AWSProperty):
    """
    `DynamoDBAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbaction.html>`__
    """

    props: PropsDictType = {
        "HashKeyField": (str, True),
        "HashKeyType": (str, False),
        "HashKeyValue": (str, True),
        "PayloadField": (str, False),
        "RangeKeyField": (str, False),
        "RangeKeyType": (str, False),
        "RangeKeyValue": (str, False),
        "RoleArn": (str, True),
        "TableName": (str, True),
    }


class PutItemInput(AWSProperty):
    """
    `PutItemInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-putiteminput.html>`__
    """

    props: PropsDictType = {
        "TableName": (str, True),
    }


class DynamoDBv2Action(AWSProperty):
    """
    `DynamoDBv2Action <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbv2action.html>`__
    """

    props: PropsDictType = {
        "PutItem": (PutItemInput, False),
        "RoleArn": (str, False),
    }


class ElasticsearchAction(AWSProperty):
    """
    `ElasticsearchAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-elasticsearchaction.html>`__
    """

    props: PropsDictType = {
        "Endpoint": (str, True),
        "Id": (str, True),
        "Index": (str, True),
        "RoleArn": (str, True),
        "Type": (str, True),
    }


class FirehoseAction(AWSProperty):
    """
    `FirehoseAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-firehoseaction.html>`__
    """

    props: PropsDictType = {
        "BatchMode": (boolean, False),
        "DeliveryStreamName": (str, True),
        "RoleArn": (str, True),
        "Separator": (str, False),
    }


class HttpActionHeader(AWSProperty):
    """
    `HttpActionHeader <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-httpactionheader.html>`__
    """

    props: PropsDictType = {
        "Key": (str, True),
        "Value": (str, True),
    }


class SigV4Authorization(AWSProperty):
    """
    `SigV4Authorization <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-sigv4authorization.html>`__
    """

    props: PropsDictType = {
        "RoleArn": (str, True),
        "ServiceName": (str, True),
        "SigningRegion": (str, True),
    }


class HttpAuthorization(AWSProperty):
    """
    `HttpAuthorization <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-httpauthorization.html>`__
    """

    props: PropsDictType = {
        "Sigv4": (SigV4Authorization, False),
    }


class HttpAction(AWSProperty):
    """
    `HttpAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-httpaction.html>`__
    """

    props: PropsDictType = {
        "Auth": (HttpAuthorization, False),
        "ConfirmationUrl": (str, False),
        "Headers": ([HttpActionHeader], False),
        "Url": (str, True),
    }


class IotAnalyticsAction(AWSProperty):
    """
    `IotAnalyticsAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-iotanalyticsaction.html>`__
    """

    props: PropsDictType = {
        "BatchMode": (boolean, False),
        "ChannelName": (str, True),
        "RoleArn": (str, True),
    }


class IotEventsAction(AWSProperty):
    """
    `IotEventsAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-ioteventsaction.html>`__
    """

    props: PropsDictType = {
        "BatchMode": (boolean, False),
        "InputName": (str, True),
        "MessageId": (str, False),
        "RoleArn": (str, True),
    }


class AssetPropertyTimestamp(AWSProperty):
    """
    `AssetPropertyTimestamp <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-assetpropertytimestamp.html>`__
    """

    props: PropsDictType = {
        "OffsetInNanos": (str, False),
        "TimeInSeconds": (str, True),
    }


class AssetPropertyVariant(AWSProperty):
    """
    `AssetPropertyVariant <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-assetpropertyvariant.html>`__
    """

    props: PropsDictType = {
        "BooleanValue": (str, False),
        "DoubleValue": (str, False),
        "IntegerValue": (str, False),
        "StringValue": (str, False),
    }


class AssetPropertyValue(AWSProperty):
    """
    `AssetPropertyValue <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-assetpropertyvalue.html>`__
    """

    props: PropsDictType = {
        "Quality": (str, False),
        "Timestamp": (AssetPropertyTimestamp, True),
        "Value": (AssetPropertyVariant, True),
    }


class PutAssetPropertyValueEntry(AWSProperty):
    """
    `PutAssetPropertyValueEntry <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-putassetpropertyvalueentry.html>`__
    """

    props: PropsDictType = {
        "AssetId": (str, False),
        "EntryId": (str, False),
        "PropertyAlias": (str, False),
        "PropertyId": (str, False),
        "PropertyValues": ([AssetPropertyValue], True),
    }


class IotSiteWiseAction(AWSProperty):
    """
    `IotSiteWiseAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-iotsitewiseaction.html>`__
    """

    props: PropsDictType = {
        "PutAssetPropertyValueEntries": ([PutAssetPropertyValueEntry], True),
        "RoleArn": (str, True),
    }


class KafkaActionHeader(AWSProperty):
    """
    `KafkaActionHeader <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-kafkaactionheader.html>`__
    """

    props: PropsDictType = {
        "Key": (str, True),
        "Value": (str, True),
    }


class KafkaAction(AWSProperty):
    """
    `KafkaAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-kafkaaction.html>`__
    """

    props: PropsDictType = {
        "ClientProperties": (dict, True),
        "DestinationArn": (str, True),
        "Headers": ([KafkaActionHeader], False),
        "Key": (str, False),
        "Partition": (str, False),
        "Topic": (str, True),
    }


class KinesisAction(AWSProperty):
    """
    `KinesisAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-kinesisaction.html>`__
    """

    props: PropsDictType = {
        "PartitionKey": (str, False),
        "RoleArn": (str, True),
        "StreamName": (str, True),
    }


class LambdaAction(AWSProperty):
    """
    `LambdaAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-lambdaaction.html>`__
    """

    props: PropsDictType = {
        "FunctionArn": (str, False),
    }


class Timestamp(AWSProperty):
    """
    `Timestamp <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-timestamp.html>`__
    """

    props: PropsDictType = {
        "Unit": (str, False),
        "Value": (str, True),
    }


class LocationAction(AWSProperty):
    """
    `LocationAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-locationaction.html>`__
    """

    props: PropsDictType = {
        "DeviceId": (str, True),
        "Latitude": (str, True),
        "Longitude": (str, True),
        "RoleArn": (str, True),
        "Timestamp": (Timestamp, False),
        "TrackerName": (str, True),
    }


class OpenSearchAction(AWSProperty):
    """
    `OpenSearchAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-opensearchaction.html>`__
    """

    props: PropsDictType = {
        "Endpoint": (str, True),
        "Id": (str, True),
        "Index": (str, True),
        "RoleArn": (str, True),
        "Type": (str, True),
    }


class UserProperty(AWSProperty):
    """
    `UserProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-userproperty.html>`__
    """

    props: PropsDictType = {
        "Key": (str, True),
        "Value": (str, True),
    }


class RepublishActionHeaders(AWSProperty):
    """
    `RepublishActionHeaders <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-republishactionheaders.html>`__
    """

    props: PropsDictType = {
        "ContentType": (str, False),
        "CorrelationData": (str, False),
        "MessageExpiry": (str, False),
        "PayloadFormatIndicator": (str, False),
        "ResponseTopic": (str, False),
        "UserProperties": ([UserProperty], False),
    }


class RepublishAction(AWSProperty):
    """
    `RepublishAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-republishaction.html>`__
    """

    props: PropsDictType = {
        "Headers": (RepublishActionHeaders, False),
        "Qos": (integer, False),
        "RoleArn": (str, True),
        "Topic": (str, True),
    }


class S3Action(AWSProperty):
    """
    `S3Action <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-s3action.html>`__
    """

    props: PropsDictType = {
        "BucketName": (str, True),
        "CannedAcl": (str, False),
        "Key": (str, True),
        "RoleArn": (str, True),
    }


class SnsAction(AWSProperty):
    """
    `SnsAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-snsaction.html>`__
    """

    props: PropsDictType = {
        "MessageFormat": (str, False),
        "RoleArn": (str, True),
        "TargetArn": (str, True),
    }


class SqsAction(AWSProperty):
    """
    `SqsAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-sqsaction.html>`__
    """

    props: PropsDictType = {
        "QueueUrl": (str, True),
        "RoleArn": (str, True),
        "UseBase64": (boolean, False),
    }


class StepFunctionsAction(AWSProperty):
    """
    `StepFunctionsAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-stepfunctionsaction.html>`__
    """

    props: PropsDictType = {
        "ExecutionNamePrefix": (str, False),
        "RoleArn": (str, True),
        "StateMachineName": (str, True),
    }


class TimestreamDimension(AWSProperty):
    """
    `TimestreamDimension <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-timestreamdimension.html>`__
    """

    props: PropsDictType = {
        "Name": (str, True),
        "Value": (str, True),
    }


class TimestreamTimestamp(AWSProperty):
    """
    `TimestreamTimestamp <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-timestreamtimestamp.html>`__
    """

    props: PropsDictType = {
        "Unit": (str, True),
        "Value": (str, True),
    }


class TimestreamAction(AWSProperty):
    """
    `TimestreamAction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-timestreamaction.html>`__
    """

    props: PropsDictType = {
        "DatabaseName": (str, True),
        "Dimensions": ([TimestreamDimension], True),
        "RoleArn": (str, True),
        "TableName": (str, True),
        "Timestamp": (TimestreamTimestamp, False),
    }


class Action(AWSProperty):
    """
    `Action <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html>`__
    """

    props: PropsDictType = {
        "CloudwatchAlarm": (CloudwatchAlarmAction, False),
        "CloudwatchLogs": (CloudwatchLogsAction, False),
        "CloudwatchMetric": (CloudwatchMetricAction, False),
        "DynamoDB": (DynamoDBAction, False),
        "DynamoDBv2": (DynamoDBv2Action, False),
        "Elasticsearch": (ElasticsearchAction, False),
        "Firehose": (FirehoseAction, False),
        "Http": (HttpAction, False),
        "IotAnalytics": (IotAnalyticsAction, False),
        "IotEvents": (IotEventsAction, False),
        "IotSiteWise": (IotSiteWiseAction, False),
        "Kafka": (KafkaAction, False),
        "Kinesis": (KinesisAction, False),
        "Lambda": (LambdaAction, False),
        "Location": (LocationAction, False),
        "OpenSearch": (OpenSearchAction, False),
        "Republish": (RepublishAction, False),
        "S3": (S3Action, False),
        "Sns": (SnsAction, False),
        "Sqs": (SqsAction, False),
        "StepFunctions": (StepFunctionsAction, False),
        "Timestream": (TimestreamAction, False),
    }


class TopicRulePayload(AWSProperty):
    """
    `TopicRulePayload <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-topicrulepayload.html>`__
    """

    props: PropsDictType = {
        "Actions": ([Action], True),
        "AwsIotSqlVersion": (str, False),
        "Description": (str, False),
        "ErrorAction": (Action, False),
        "RuleDisabled": (boolean, False),
        "Sql": (str, True),
    }


class TopicRule(AWSObject):
    """
    `TopicRule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-topicrule.html>`__
    """

    resource_type = "AWS::IoT::TopicRule"

    props: PropsDictType = {
        "RuleName": (str, False),
        "Tags": (Tags, False),
        "TopicRulePayload": (TopicRulePayload, True),
    }


class HttpUrlDestinationSummary(AWSProperty):
    """
    `HttpUrlDestinationSummary <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicruledestination-httpurldestinationsummary.html>`__
    """

    props: PropsDictType = {
        "ConfirmationUrl": (str, False),
    }


class VpcDestinationProperties(AWSProperty):
    """
    `VpcDestinationProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicruledestination-vpcdestinationproperties.html>`__
    """

    props: PropsDictType = {
        "RoleArn": (str, False),
        "SecurityGroups": ([str], False),
        "SubnetIds": ([str], False),
        "VpcId": (str, False),
    }


class TopicRuleDestination(AWSObject):
    """
    `TopicRuleDestination <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-topicruledestination.html>`__
    """

    resource_type = "AWS::IoT::TopicRuleDestination"

    props: PropsDictType = {
        "HttpUrlProperties": (HttpUrlDestinationSummary, False),
        "Status": (str, False),
        "VpcProperties": (VpcDestinationProperties, False),
    }


class ServerCertificateSummary(AWSProperty):
    """
    `ServerCertificateSummary <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-domainconfiguration-servercertificatesummary.html>`__
    """

    props: PropsDictType = {
        "ServerCertificateArn": (str, False),
        "ServerCertificateStatus": (str, False),
        "ServerCertificateStatusDetail": (str, False),
    }



from troposphere import Template
from troposphere.codedeploy import (
    AutoRollbackConfiguration,
    DeploymentGroup,
    DeploymentStyle,
    Ec2TagFilters,
    Ec2TagSet,
    Ec2TagSetListObject,
    ElbInfoList,
    LoadBalancerInfo,
    OnPremisesInstanceTagFilters,
)

template = Template()
template.set_version("2010-09-09")


auto_rollback_configuration = AutoRollbackConfiguration(
    Enabled=True, Events=["DEPLOYMENT_FAILURE"]
)

deployment_style = DeploymentStyle(DeploymentOption="WITH_TRAFFIC_CONTROL")

elb_info_list = ElbInfoList(Name="DemoLoadBalancer")

load_balancer_info = LoadBalancerInfo(ElbInfoList=[elb_info_list])

deployment_group = DeploymentGroup(
    "DemoDeploymentGroup",
    ApplicationName="DemoApplication",
    AutoRollbackConfiguration=auto_rollback_configuration,
    DeploymentStyle=deployment_style,
    LoadBalancerInfo=load_balancer_info,
    ServiceRoleArn="arn:aws:iam::0123456789:role/codedeploy-role",
)
template.add_resource(deployment_group)

# EC2 instances
deployment_group_ec2 = DeploymentGroup(
    "DemoDeploymentGroupEC2Instances",
    DeploymentGroupName="DemoApplicationEc2Instances",
    ApplicationName="DemoApplicationEc2Instances",
    AutoRollbackConfiguration=auto_rollback_configuration,
    DeploymentStyle=DeploymentStyle(DeploymentOption="WITHOUT_TRAFFIC_CONTROL"),
    ServiceRoleArn="arn:aws:iam::0123456789:role/codedeploy-role",
    Ec2TagSet=Ec2TagSet(
        Ec2TagSetList=[
            Ec2TagSetListObject(
                Ec2TagGroup=[
                    Ec2TagFilters(
                        Key="CodeDeploy", Type="KEY_AND_VALUE", Value="activated"
                    ),
                    Ec2TagFilters(Key="Environment", Type="KEY_AND_VALUE", Value="dev"),
                ]
            )
        ]
    ),
)

template.add_resource(deployment_group_ec2)


# On premises
deployment_group_on_premises = DeploymentGroup(
    "DemoDeploymentGroupOnPremises",
    DeploymentGroupName="DemoApplicationOnPremises",
    ApplicationName="DemoApplicationOnPremises",
    AutoRollbackConfiguration=auto_rollback_configuration,
    DeploymentStyle=DeploymentStyle(DeploymentOption="WITHOUT_TRAFFIC_CONTROL"),
    ServiceRoleArn="arn:aws:iam::0123456789:role/codedeploy-role",
    OnPremisesInstanceTagFilters=[
        OnPremisesInstanceTagFilters(
            Key="Service", Value="DemoApplicationOnPremises", Type="KEY_AND_VALUE"
        )
    ],
)
template.add_resource(deployment_group_on_premises)

print(template.to_json())



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer
from .validators.iam import Active  # noqa: F401
from .validators.iam import Inactive  # noqa: F401
from .validators.iam import (
    iam_group_name,
    iam_path,
    iam_role_name,
    iam_user_name,
    policytypes,
    status,
    validate_tags_or_list,
)


class AccessKey(AWSObject):
    """
    `AccessKey <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html>`__
    """

    resource_type = "AWS::IAM::AccessKey"

    props: PropsDictType = {
        "Serial": (integer, False),
        "Status": (status, False),
        "UserName": (str, True),
    }


class Policy(AWSProperty):
    """
    `Policy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user-policy.html>`__
    """

    props: PropsDictType = {
        "PolicyDocument": (policytypes, True),
        "PolicyName": (str, True),
    }


class Group(AWSObject):
    """
    `Group <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-group.html>`__
    """

    resource_type = "AWS::IAM::Group"

    props: PropsDictType = {
        "GroupName": (iam_group_name, False),
        "ManagedPolicyArns": ([str], False),
        "Path": (iam_path, False),
        "Policies": ([Policy], False),
    }


class GroupPolicy(AWSObject):
    """
    `GroupPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-grouppolicy.html>`__
    """

    resource_type = "AWS::IAM::GroupPolicy"

    props: PropsDictType = {
        "GroupName": (str, True),
        "PolicyDocument": (dict, False),
        "PolicyName": (str, True),
    }


class InstanceProfile(AWSObject):
    """
    `InstanceProfile <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html>`__
    """

    resource_type = "AWS::IAM::InstanceProfile"

    props: PropsDictType = {
        "InstanceProfileName": (str, False),
        "Path": (iam_path, False),
        "Roles": ([str], True),
    }


class ManagedPolicy(AWSObject):
    """
    `ManagedPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-managedpolicy.html>`__
    """

    resource_type = "AWS::IAM::ManagedPolicy"

    props: PropsDictType = {
        "Description": (str, False),
        "Groups": ([str], False),
        "ManagedPolicyName": (str, False),
        "Path": (iam_path, False),
        "PolicyDocument": (policytypes, True),
        "Roles": ([str], False),
        "Users": ([str], False),
    }


class OIDCProvider(AWSObject):
    """
    `OIDCProvider <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-oidcprovider.html>`__
    """

    resource_type = "AWS::IAM::OIDCProvider"

    props: PropsDictType = {
        "ClientIdList": ([str], False),
        "Tags": (Tags, False),
        "ThumbprintList": ([str], True),
        "Url": (str, False),
    }


class PolicyType(AWSObject):
    """
    `PolicyType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html>`__
    """

    resource_type = "AWS::IAM::Policy"

    props: PropsDictType = {
        "Groups": ([str], False),
        "PolicyDocument": (policytypes, True),
        "PolicyName": (str, True),
        "Roles": ([str], False),
        "Users": ([str], False),
    }


class Role(AWSObject):
    """
    `Role <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html>`__
    """

    resource_type = "AWS::IAM::Role"

    props: PropsDictType = {
        "AssumeRolePolicyDocument": (policytypes, True),
        "Description": (str, False),
        "ManagedPolicyArns": ([str], False),
        "MaxSessionDuration": (integer, False),
        "Path": (iam_path, False),
        "PermissionsBoundary": (str, False),
        "Policies": ([Policy], False),
        "RoleName": (iam_role_name, False),
        "Tags": (validate_tags_or_list, False),
    }


class RolePolicy(AWSObject):
    """
    `RolePolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-rolepolicy.html>`__
    """

    resource_type = "AWS::IAM::RolePolicy"

    props: PropsDictType = {
        "PolicyDocument": (dict, False),
        "PolicyName": (str, True),
        "RoleName": (str, True),
    }


class SAMLProvider(AWSObject):
    """
    `SAMLProvider <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-samlprovider.html>`__
    """

    resource_type = "AWS::IAM::SAMLProvider"

    props: PropsDictType = {
        "Name": (str, False),
        "SamlMetadataDocument": (str, True),
        "Tags": (Tags, False),
    }


class ServerCertificate(AWSObject):
    """
    `ServerCertificate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-servercertificate.html>`__
    """

    resource_type = "AWS::IAM::ServerCertificate"

    props: PropsDictType = {
        "CertificateBody": (str, False),
        "CertificateChain": (str, False),
        "Path": (str, False),
        "PrivateKey": (str, False),
        "ServerCertificateName": (str, False),
        "Tags": (Tags, False),
    }


class ServiceLinkedRole(AWSObject):
    """
    `ServiceLinkedRole <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-servicelinkedrole.html>`__
    """

    resource_type = "AWS::IAM::ServiceLinkedRole"

    props: PropsDictType = {
        "AWSServiceName": (str, False),
        "CustomSuffix": (str, False),
        "Description": (str, False),
    }


class LoginProfile(AWSProperty):
    """
    `LoginProfile <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user-loginprofile.html>`__
    """

    props: PropsDictType = {
        "Password": (str, True),
        "PasswordResetRequired": (boolean, False),
    }


class User(AWSObject):
    """
    `User <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-user.html>`__
    """

    resource_type = "AWS::IAM::User"

    props: PropsDictType = {
        "Groups": ([str], False),
        "LoginProfile": (LoginProfile, False),
        "ManagedPolicyArns": ([str], False),
        "Path": (iam_path, False),
        "PermissionsBoundary": (str, False),
        "Policies": ([Policy], False),
        "Tags": (Tags, False),
        "UserName": (iam_user_name, False),
    }


class UserPolicy(AWSObject):
    """
    `UserPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-userpolicy.html>`__
    """

    resource_type = "AWS::IAM::UserPolicy"

    props: PropsDictType = {
        "PolicyDocument": (dict, False),
        "PolicyName": (str, True),
        "UserName": (str, True),
    }


class UserToGroupAddition(AWSObject):
    """
    `UserToGroupAddition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-addusertogroup.html>`__
    """

    resource_type = "AWS::IAM::UserToGroupAddition"

    props: PropsDictType = {
        "GroupName": (str, True),
        "Users": ([str], True),
    }


class VirtualMFADevice(AWSObject):
    """
    `VirtualMFADevice <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-virtualmfadevice.html>`__
    """

    resource_type = "AWS::IAM::VirtualMFADevice"

    props: PropsDictType = {
        "Path": (str, False),
        "Tags": (Tags, False),
        "Users": ([str], True),
        "VirtualMfaDeviceName": (str, False),
    }



from . import AWSAttribute, AWSProperty, PropsDictType, validate_pausetime
from .validators import boolean, integer, positive_integer


class AutoScalingRollingUpdate(AWSProperty):
    props: PropsDictType = {
        "MaxBatchSize": (positive_integer, False),
        "MinInstancesInService": (integer, False),
        "MinSuccessfulInstancesPercent": (integer, False),
        "PauseTime": (validate_pausetime, False),
        "SuspendProcesses": ([str], False),
        "WaitOnResourceSignals": (boolean, False),
    }


class AutoScalingScheduledAction(AWSProperty):
    props: PropsDictType = {
        "IgnoreUnmodifiedGroupSizeProperties": (boolean, False),
    }


class AutoScalingReplacingUpdate(AWSProperty):
    props: PropsDictType = {
        "WillReplace": (boolean, False),
    }


class CodeDeployLambdaAliasUpdate(AWSProperty):
    props: PropsDictType = {
        "AfterAllowTrafficHook": (str, False),
        "ApplicationName": (boolean, True),
        "BeforeAllowTrafficHook": (str, False),
        "DeploymentGroupName": (boolean, True),
    }


class UpdatePolicy(AWSAttribute):
    props: PropsDictType = {
        "AutoScalingRollingUpdate": (AutoScalingRollingUpdate, False),
        "AutoScalingScheduledAction": (AutoScalingScheduledAction, False),
        "AutoScalingReplacingUpdate": (AutoScalingReplacingUpdate, False),
        "CodeDeployLambdaAliasUpdate": (CodeDeployLambdaAliasUpdate, False),
        "UseOnlineResharding": (boolean, False),
        "EnableVersionUpgrade": (boolean, False),
    }


class ResourceSignal(AWSProperty):
    props: PropsDictType = {
        "Count": (positive_integer, False),
        "Timeout": (validate_pausetime, False),
    }


class AutoScalingCreationPolicy(AWSProperty):
    props: PropsDictType = {
        "MinSuccessfulInstancesPercent": (integer, False),
    }


class CreationPolicy(AWSAttribute):
    props: PropsDictType = {
        "AutoScalingCreationPolicy": (AutoScalingCreationPolicy, False),
        "ResourceSignal": (ResourceSignal, True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer
from .validators.kinesisanalyticsv2 import validate_runtime_environment


class S3ContentLocation(AWSProperty):
    """
    `S3ContentLocation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-s3contentlocation.html>`__
    """

    props: PropsDictType = {
        "BucketARN": (str, True),
        "FileKey": (str, True),
        "ObjectVersion": (str, False),
    }


class CodeContent(AWSProperty):
    """
    `CodeContent <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-codecontent.html>`__
    """

    props: PropsDictType = {
        "S3ContentLocation": (S3ContentLocation, False),
        "TextContent": (str, False),
        "ZipFileContent": (str, False),
    }


class ApplicationCodeConfiguration(AWSProperty):
    """
    `ApplicationCodeConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-applicationcodeconfiguration.html>`__
    """

    props: PropsDictType = {
        "CodeContent": (CodeContent, True),
        "CodeContentType": (str, True),
    }


class ApplicationSnapshotConfiguration(AWSProperty):
    """
    `ApplicationSnapshotConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-applicationsnapshotconfiguration.html>`__
    """

    props: PropsDictType = {
        "SnapshotsEnabled": (boolean, True),
    }


class PropertyGroup(AWSProperty):
    """
    `PropertyGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-propertygroup.html>`__
    """

    props: PropsDictType = {
        "PropertyGroupId": (str, False),
        "PropertyMap": (dict, False),
    }


class EnvironmentProperties(AWSProperty):
    """
    `EnvironmentProperties <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-environmentproperties.html>`__
    """

    props: PropsDictType = {
        "PropertyGroups": ([PropertyGroup], False),
    }


class CheckpointConfiguration(AWSProperty):
    """
    `CheckpointConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-checkpointconfiguration.html>`__
    """

    props: PropsDictType = {
        "CheckpointInterval": (integer, False),
        "CheckpointingEnabled": (boolean, False),
        "ConfigurationType": (str, True),
        "MinPauseBetweenCheckpoints": (integer, False),
    }


class MonitoringConfiguration(AWSProperty):
    """
    `MonitoringConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-monitoringconfiguration.html>`__
    """

    props: PropsDictType = {
        "ConfigurationType": (str, True),
        "LogLevel": (str, False),
        "MetricsLevel": (str, False),
    }


class ParallelismConfiguration(AWSProperty):
    """
    `ParallelismConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-parallelismconfiguration.html>`__
    """

    props: PropsDictType = {
        "AutoScalingEnabled": (boolean, False),
        "ConfigurationType": (str, True),
        "Parallelism": (integer, False),
        "ParallelismPerKPU": (integer, False),
    }


class FlinkApplicationConfiguration(AWSProperty):
    """
    `FlinkApplicationConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-flinkapplicationconfiguration.html>`__
    """

    props: PropsDictType = {
        "CheckpointConfiguration": (CheckpointConfiguration, False),
        "MonitoringConfiguration": (MonitoringConfiguration, False),
        "ParallelismConfiguration": (ParallelismConfiguration, False),
    }


class InputParallelism(AWSProperty):
    """
    `InputParallelism <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-inputparallelism.html>`__
    """

    props: PropsDictType = {
        "Count": (integer, False),
    }


class InputLambdaProcessor(AWSProperty):
    """
    `InputLambdaProcessor <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-inputlambdaprocessor.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
    }


class InputProcessingConfiguration(AWSProperty):
    """
    `InputProcessingConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-inputprocessingconfiguration.html>`__
    """

    props: PropsDictType = {
        "InputLambdaProcessor": (InputLambdaProcessor, False),
    }


class RecordColumn(AWSProperty):
    """
    `RecordColumn <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationreferencedatasource-recordcolumn.html>`__
    """

    props: PropsDictType = {
        "Mapping": (str, False),
        "Name": (str, True),
        "SqlType": (str, True),
    }


class CSVMappingParameters(AWSProperty):
    """
    `CSVMappingParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationreferencedatasource-csvmappingparameters.html>`__
    """

    props: PropsDictType = {
        "RecordColumnDelimiter": (str, True),
        "RecordRowDelimiter": (str, True),
    }


class JSONMappingParameters(AWSProperty):
    """
    `JSONMappingParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationreferencedatasource-jsonmappingparameters.html>`__
    """

    props: PropsDictType = {
        "RecordRowPath": (str, True),
    }


class MappingParameters(AWSProperty):
    """
    `MappingParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationreferencedatasource-mappingparameters.html>`__
    """

    props: PropsDictType = {
        "CSVMappingParameters": (CSVMappingParameters, False),
        "JSONMappingParameters": (JSONMappingParameters, False),
    }


class RecordFormat(AWSProperty):
    """
    `RecordFormat <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationreferencedatasource-recordformat.html>`__
    """

    props: PropsDictType = {
        "MappingParameters": (MappingParameters, False),
        "RecordFormatType": (str, True),
    }


class InputSchema(AWSProperty):
    """
    `InputSchema <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-inputschema.html>`__
    """

    props: PropsDictType = {
        "RecordColumns": ([RecordColumn], True),
        "RecordEncoding": (str, False),
        "RecordFormat": (RecordFormat, True),
    }


class KinesisFirehoseInput(AWSProperty):
    """
    `KinesisFirehoseInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-kinesisfirehoseinput.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
    }


class KinesisStreamsInput(AWSProperty):
    """
    `KinesisStreamsInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-kinesisstreamsinput.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
    }


class Input(AWSProperty):
    """
    `Input <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-input.html>`__
    """

    props: PropsDictType = {
        "InputParallelism": (InputParallelism, False),
        "InputProcessingConfiguration": (InputProcessingConfiguration, False),
        "InputSchema": (InputSchema, True),
        "KinesisFirehoseInput": (KinesisFirehoseInput, False),
        "KinesisStreamsInput": (KinesisStreamsInput, False),
        "NamePrefix": (str, True),
    }


class SqlApplicationConfiguration(AWSProperty):
    """
    `SqlApplicationConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-sqlapplicationconfiguration.html>`__
    """

    props: PropsDictType = {
        "Inputs": ([Input], False),
    }


class VpcConfiguration(AWSProperty):
    """
    `VpcConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-vpcconfiguration.html>`__
    """

    props: PropsDictType = {
        "SecurityGroupIds": ([str], True),
        "SubnetIds": ([str], True),
    }


class GlueDataCatalogConfiguration(AWSProperty):
    """
    `GlueDataCatalogConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-gluedatacatalogconfiguration.html>`__
    """

    props: PropsDictType = {
        "DatabaseARN": (str, False),
    }


class CatalogConfiguration(AWSProperty):
    """
    `CatalogConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-catalogconfiguration.html>`__
    """

    props: PropsDictType = {
        "GlueDataCatalogConfiguration": (GlueDataCatalogConfiguration, False),
    }


class MavenReference(AWSProperty):
    """
    `MavenReference <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-mavenreference.html>`__
    """

    props: PropsDictType = {
        "ArtifactId": (str, True),
        "GroupId": (str, True),
        "Version": (str, True),
    }


class CustomArtifactConfiguration(AWSProperty):
    """
    `CustomArtifactConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-customartifactconfiguration.html>`__
    """

    props: PropsDictType = {
        "ArtifactType": (str, True),
        "MavenReference": (MavenReference, False),
        "S3ContentLocation": (S3ContentLocation, False),
    }


class S3ContentBaseLocation(AWSProperty):
    """
    `S3ContentBaseLocation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-s3contentbaselocation.html>`__
    """

    props: PropsDictType = {
        "BasePath": (str, False),
        "BucketARN": (str, True),
    }


class DeployAsApplicationConfiguration(AWSProperty):
    """
    `DeployAsApplicationConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-deployasapplicationconfiguration.html>`__
    """

    props: PropsDictType = {
        "S3ContentLocation": (S3ContentBaseLocation, True),
    }


class ZeppelinMonitoringConfiguration(AWSProperty):
    """
    `ZeppelinMonitoringConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-zeppelinmonitoringconfiguration.html>`__
    """

    props: PropsDictType = {
        "LogLevel": (str, False),
    }


class ZeppelinApplicationConfiguration(AWSProperty):
    """
    `ZeppelinApplicationConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-zeppelinapplicationconfiguration.html>`__
    """

    props: PropsDictType = {
        "CatalogConfiguration": (CatalogConfiguration, False),
        "CustomArtifactsConfiguration": ([CustomArtifactConfiguration], False),
        "DeployAsApplicationConfiguration": (DeployAsApplicationConfiguration, False),
        "MonitoringConfiguration": (ZeppelinMonitoringConfiguration, False),
    }


class ApplicationConfiguration(AWSProperty):
    """
    `ApplicationConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-applicationconfiguration.html>`__
    """

    props: PropsDictType = {
        "ApplicationCodeConfiguration": (ApplicationCodeConfiguration, False),
        "ApplicationSnapshotConfiguration": (ApplicationSnapshotConfiguration, False),
        "EnvironmentProperties": (EnvironmentProperties, False),
        "FlinkApplicationConfiguration": (FlinkApplicationConfiguration, False),
        "SqlApplicationConfiguration": (SqlApplicationConfiguration, False),
        "VpcConfigurations": ([VpcConfiguration], False),
        "ZeppelinApplicationConfiguration": (ZeppelinApplicationConfiguration, False),
    }


class ApplicationMaintenanceConfiguration(AWSProperty):
    """
    `ApplicationMaintenanceConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-applicationmaintenanceconfiguration.html>`__
    """

    props: PropsDictType = {
        "ApplicationMaintenanceWindowStartTime": (str, True),
    }


class ApplicationRestoreConfiguration(AWSProperty):
    """
    `ApplicationRestoreConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-applicationrestoreconfiguration.html>`__
    """

    props: PropsDictType = {
        "ApplicationRestoreType": (str, True),
        "SnapshotName": (str, False),
    }


class FlinkRunConfiguration(AWSProperty):
    """
    `FlinkRunConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-flinkrunconfiguration.html>`__
    """

    props: PropsDictType = {
        "AllowNonRestoredState": (boolean, False),
    }


class RunConfiguration(AWSProperty):
    """
    `RunConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-application-runconfiguration.html>`__
    """

    props: PropsDictType = {
        "ApplicationRestoreConfiguration": (ApplicationRestoreConfiguration, False),
        "FlinkRunConfiguration": (FlinkRunConfiguration, False),
    }


class Application(AWSObject):
    """
    `Application <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalyticsv2-application.html>`__
    """

    resource_type = "AWS::KinesisAnalyticsV2::Application"

    props: PropsDictType = {
        "ApplicationConfiguration": (ApplicationConfiguration, False),
        "ApplicationDescription": (str, False),
        "ApplicationMaintenanceConfiguration": (
            ApplicationMaintenanceConfiguration,
            False,
        ),
        "ApplicationMode": (str, False),
        "ApplicationName": (str, False),
        "RunConfiguration": (RunConfiguration, False),
        "RuntimeEnvironment": (validate_runtime_environment, True),
        "ServiceExecutionRole": (str, True),
        "Tags": (Tags, False),
    }


class CloudWatchLoggingOption(AWSProperty):
    """
    `CloudWatchLoggingOption <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationcloudwatchloggingoption-cloudwatchloggingoption.html>`__
    """

    props: PropsDictType = {
        "LogStreamARN": (str, True),
    }


class ApplicationCloudWatchLoggingOption(AWSObject):
    """
    `ApplicationCloudWatchLoggingOption <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalyticsv2-applicationcloudwatchloggingoption.html>`__
    """

    resource_type = "AWS::KinesisAnalyticsV2::ApplicationCloudWatchLoggingOption"

    props: PropsDictType = {
        "ApplicationName": (str, True),
        "CloudWatchLoggingOption": (CloudWatchLoggingOption, True),
    }


class DestinationSchema(AWSProperty):
    """
    `DestinationSchema <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationoutput-destinationschema.html>`__
    """

    props: PropsDictType = {
        "RecordFormatType": (str, False),
    }


class KinesisFirehoseOutput(AWSProperty):
    """
    `KinesisFirehoseOutput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationoutput-kinesisfirehoseoutput.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
    }


class KinesisStreamsOutput(AWSProperty):
    """
    `KinesisStreamsOutput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationoutput-kinesisstreamsoutput.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
    }


class LambdaOutput(AWSProperty):
    """
    `LambdaOutput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationoutput-lambdaoutput.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
    }


class Output(AWSProperty):
    """
    `Output <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationoutput-output.html>`__
    """

    props: PropsDictType = {
        "DestinationSchema": (DestinationSchema, True),
        "KinesisFirehoseOutput": (KinesisFirehoseOutput, False),
        "KinesisStreamsOutput": (KinesisStreamsOutput, False),
        "LambdaOutput": (LambdaOutput, False),
        "Name": (str, False),
    }


class ApplicationOutput(AWSObject):
    """
    `ApplicationOutput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalyticsv2-applicationoutput.html>`__
    """

    resource_type = "AWS::KinesisAnalyticsV2::ApplicationOutput"

    props: PropsDictType = {
        "ApplicationName": (str, True),
        "Output": (Output, True),
    }


class ReferenceSchema(AWSProperty):
    """
    `ReferenceSchema <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationreferencedatasource-referenceschema.html>`__
    """

    props: PropsDictType = {
        "RecordColumns": ([RecordColumn], True),
        "RecordEncoding": (str, False),
        "RecordFormat": (RecordFormat, True),
    }


class S3ReferenceDataSource(AWSProperty):
    """
    `S3ReferenceDataSource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationreferencedatasource-s3referencedatasource.html>`__
    """

    props: PropsDictType = {
        "BucketARN": (str, True),
        "FileKey": (str, True),
    }


class ReferenceDataSource(AWSProperty):
    """
    `ReferenceDataSource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalyticsv2-applicationreferencedatasource-referencedatasource.html>`__
    """

    props: PropsDictType = {
        "ReferenceSchema": (ReferenceSchema, True),
        "S3ReferenceDataSource": (S3ReferenceDataSource, False),
        "TableName": (str, False),
    }


class ApplicationReferenceDataSource(AWSObject):
    """
    `ApplicationReferenceDataSource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalyticsv2-applicationreferencedatasource.html>`__
    """

    resource_type = "AWS::KinesisAnalyticsV2::ApplicationReferenceDataSource"

    props: PropsDictType = {
        "ApplicationName": (str, True),
        "ReferenceDataSource": (ReferenceDataSource, True),
    }



import unittest

from troposphere import Name
from troposphere.iottwinmaker import DataType, DataValue


class TestPlacementTemplate(unittest.TestCase):
    def test_datatype_nestedtype(self):
        with self.assertRaises(TypeError):
            DataType(NestedType="foo")

        DataType(NestedType=DataType())
        DataType(NestedType=Name("foo"))

    def test_datavalue_listvalue(self):
        with self.assertRaises(TypeError):
            DataValue(ListValue="foo")

        with self.assertRaises(TypeError):
            DataValue(ListValue=["foo"])

        DataValue(ListValue=[DataValue(), Name("foo")])



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType


class Group(AWSObject):
    """
    `Group <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-identitystore-group.html>`__
    """

    resource_type = "AWS::IdentityStore::Group"

    props: PropsDictType = {
        "Description": (str, False),
        "DisplayName": (str, True),
        "IdentityStoreId": (str, True),
    }


class MemberId(AWSProperty):
    """
    `MemberId <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-identitystore-groupmembership-memberid.html>`__
    """

    props: PropsDictType = {
        "UserId": (str, True),
    }


class GroupMembership(AWSObject):
    """
    `GroupMembership <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-identitystore-groupmembership.html>`__
    """

    resource_type = "AWS::IdentityStore::GroupMembership"

    props: PropsDictType = {
        "GroupId": (str, True),
        "IdentityStoreId": (str, True),
        "MemberId": (MemberId, True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from ..type_defs.compat import Final

ONE_HOUR: Final = "One_Hour"
THREE_HOURS: Final = "Three_Hours"
SIX_HOURS: Final = "Six_Hours"
TWELVE_HOURS: Final = "Twelve_Hours"
TWENTYFOUR_HOURS: Final = "TwentyFour_Hours"


def validate_source_details(self):
    """
    Class: SourceDetails
    """

    valid_freqs = [
        ONE_HOUR,
        THREE_HOURS,
        SIX_HOURS,
        TWELVE_HOURS,
        TWENTYFOUR_HOURS,
    ]
    freq = self.properties.get("MaximumExecutionFrequency")
    if freq and freq not in valid_freqs:
        raise ValueError(
            "MaximumExecutionFrequency (given: %s) must be one of: %s"
            % (freq, ", ".join(valid_freqs))
        )



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


def validate_listvalue(values):
    """
    Property: DataValue.ListValue
    """
    from .. import AWSHelperFn
    from ..iottwinmaker import DataValue

    if not isinstance(values, list):
        raise TypeError("ListValue must be a list", list)

    for v in values:
        if not isinstance(v, (DataValue, AWSHelperFn)):
            raise TypeError(
                "ListValue must contain DataValue or AWSHelperFn",
                AWSHelperFn,
                DataValue,
            )


def validate_nestedtypel(value):
    """
    Property: DataType.NestedType
    """
    from .. import AWSHelperFn
    from ..iottwinmaker import DataType

    if not isinstance(value, (DataType, AWSHelperFn)):
        raise TypeError(
            "NestedType must be either DataType or AWSHelperFn", AWSHelperFn, DataType
        )



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import boolean, double


class VpcInformation(AWSProperty):
    """
    `VpcInformation <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-connector-vpcinformation.html>`__
    """

    props: PropsDictType = {
        "SecurityGroupIds": ([str], True),
    }


class Connector(AWSObject):
    """
    `Connector <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-pcaconnectorad-connector.html>`__
    """

    resource_type = "AWS::PCAConnectorAD::Connector"

    props: PropsDictType = {
        "CertificateAuthorityArn": (str, True),
        "DirectoryId": (str, True),
        "Tags": (dict, False),
        "VpcInformation": (VpcInformation, True),
    }


class DirectoryRegistration(AWSObject):
    """
    `DirectoryRegistration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-pcaconnectorad-directoryregistration.html>`__
    """

    resource_type = "AWS::PCAConnectorAD::DirectoryRegistration"

    props: PropsDictType = {
        "DirectoryId": (str, True),
        "Tags": (dict, False),
    }


class ServicePrincipalName(AWSObject):
    """
    `ServicePrincipalName <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-pcaconnectorad-serviceprincipalname.html>`__
    """

    resource_type = "AWS::PCAConnectorAD::ServicePrincipalName"

    props: PropsDictType = {
        "ConnectorArn": (str, False),
        "DirectoryRegistrationArn": (str, False),
    }


class ValidityPeriod(AWSProperty):
    """
    `ValidityPeriod <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-validityperiod.html>`__
    """

    props: PropsDictType = {
        "Period": (double, True),
        "PeriodType": (str, True),
    }


class CertificateValidity(AWSProperty):
    """
    `CertificateValidity <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-certificatevalidity.html>`__
    """

    props: PropsDictType = {
        "RenewalPeriod": (ValidityPeriod, True),
        "ValidityPeriod": (ValidityPeriod, True),
    }


class EnrollmentFlagsV2(AWSProperty):
    """
    `EnrollmentFlagsV2 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-enrollmentflagsv2.html>`__
    """

    props: PropsDictType = {
        "EnableKeyReuseOnNtTokenKeysetStorageFull": (boolean, False),
        "IncludeSymmetricAlgorithms": (boolean, False),
        "NoSecurityExtension": (boolean, False),
        "RemoveInvalidCertificateFromPersonalStore": (boolean, False),
        "UserInteractionRequired": (boolean, False),
    }


class ApplicationPolicy(AWSProperty):
    """
    `ApplicationPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-applicationpolicy.html>`__
    """

    props: PropsDictType = {
        "PolicyObjectIdentifier": (str, False),
        "PolicyType": (str, False),
    }


class ApplicationPolicies(AWSProperty):
    """
    `ApplicationPolicies <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-applicationpolicies.html>`__
    """

    props: PropsDictType = {
        "Critical": (boolean, False),
        "Policies": ([ApplicationPolicy], True),
    }


class KeyUsageFlags(AWSProperty):
    """
    `KeyUsageFlags <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-keyusageflags.html>`__
    """

    props: PropsDictType = {
        "DataEncipherment": (boolean, False),
        "DigitalSignature": (boolean, False),
        "KeyAgreement": (boolean, False),
        "KeyEncipherment": (boolean, False),
        "NonRepudiation": (boolean, False),
    }


class KeyUsage(AWSProperty):
    """
    `KeyUsage <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-keyusage.html>`__
    """

    props: PropsDictType = {
        "Critical": (boolean, False),
        "UsageFlags": (KeyUsageFlags, True),
    }


class ExtensionsV2(AWSProperty):
    """
    `ExtensionsV2 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-extensionsv2.html>`__
    """

    props: PropsDictType = {
        "ApplicationPolicies": (ApplicationPolicies, False),
        "KeyUsage": (KeyUsage, True),
    }


class GeneralFlagsV2(AWSProperty):
    """
    `GeneralFlagsV2 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-generalflagsv2.html>`__
    """

    props: PropsDictType = {
        "AutoEnrollment": (boolean, False),
        "MachineType": (boolean, False),
    }


class PrivateKeyAttributesV2(AWSProperty):
    """
    `PrivateKeyAttributesV2 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-privatekeyattributesv2.html>`__
    """

    props: PropsDictType = {
        "CryptoProviders": ([str], False),
        "KeySpec": (str, True),
        "MinimalKeyLength": (double, True),
    }


class PrivateKeyFlagsV2(AWSProperty):
    """
    `PrivateKeyFlagsV2 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-privatekeyflagsv2.html>`__
    """

    props: PropsDictType = {
        "ClientVersion": (str, True),
        "ExportableKey": (boolean, False),
        "StrongKeyProtectionRequired": (boolean, False),
    }


class SubjectNameFlagsV2(AWSProperty):
    """
    `SubjectNameFlagsV2 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-subjectnameflagsv2.html>`__
    """

    props: PropsDictType = {
        "RequireCommonName": (boolean, False),
        "RequireDirectoryPath": (boolean, False),
        "RequireDnsAsCn": (boolean, False),
        "RequireEmail": (boolean, False),
        "SanRequireDirectoryGuid": (boolean, False),
        "SanRequireDns": (boolean, False),
        "SanRequireDomainDns": (boolean, False),
        "SanRequireEmail": (boolean, False),
        "SanRequireSpn": (boolean, False),
        "SanRequireUpn": (boolean, False),
    }


class TemplateV2(AWSProperty):
    """
    `TemplateV2 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-templatev2.html>`__
    """

    props: PropsDictType = {
        "CertificateValidity": (CertificateValidity, True),
        "EnrollmentFlags": (EnrollmentFlagsV2, True),
        "Extensions": (ExtensionsV2, True),
        "GeneralFlags": (GeneralFlagsV2, True),
        "PrivateKeyAttributes": (PrivateKeyAttributesV2, True),
        "PrivateKeyFlags": (PrivateKeyFlagsV2, True),
        "SubjectNameFlags": (SubjectNameFlagsV2, True),
        "SupersededTemplates": ([str], False),
    }


class EnrollmentFlagsV3(AWSProperty):
    """
    `EnrollmentFlagsV3 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-enrollmentflagsv3.html>`__
    """

    props: PropsDictType = {
        "EnableKeyReuseOnNtTokenKeysetStorageFull": (boolean, False),
        "IncludeSymmetricAlgorithms": (boolean, False),
        "NoSecurityExtension": (boolean, False),
        "RemoveInvalidCertificateFromPersonalStore": (boolean, False),
        "UserInteractionRequired": (boolean, False),
    }


class ExtensionsV3(AWSProperty):
    """
    `ExtensionsV3 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-extensionsv3.html>`__
    """

    props: PropsDictType = {
        "ApplicationPolicies": (ApplicationPolicies, False),
        "KeyUsage": (KeyUsage, True),
    }


class GeneralFlagsV3(AWSProperty):
    """
    `GeneralFlagsV3 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-generalflagsv3.html>`__
    """

    props: PropsDictType = {
        "AutoEnrollment": (boolean, False),
        "MachineType": (boolean, False),
    }


class KeyUsagePropertyFlags(AWSProperty):
    """
    `KeyUsagePropertyFlags <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-keyusagepropertyflags.html>`__
    """

    props: PropsDictType = {
        "Decrypt": (boolean, False),
        "KeyAgreement": (boolean, False),
        "Sign": (boolean, False),
    }


class KeyUsageProperty(AWSProperty):
    """
    `KeyUsageProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-keyusageproperty.html>`__
    """

    props: PropsDictType = {
        "PropertyFlags": (KeyUsagePropertyFlags, False),
        "PropertyType": (str, False),
    }


class PrivateKeyAttributesV3(AWSProperty):
    """
    `PrivateKeyAttributesV3 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-privatekeyattributesv3.html>`__
    """

    props: PropsDictType = {
        "Algorithm": (str, True),
        "CryptoProviders": ([str], False),
        "KeySpec": (str, True),
        "KeyUsageProperty": (KeyUsageProperty, True),
        "MinimalKeyLength": (double, True),
    }


class PrivateKeyFlagsV3(AWSProperty):
    """
    `PrivateKeyFlagsV3 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-privatekeyflagsv3.html>`__
    """

    props: PropsDictType = {
        "ClientVersion": (str, True),
        "ExportableKey": (boolean, False),
        "RequireAlternateSignatureAlgorithm": (boolean, False),
        "StrongKeyProtectionRequired": (boolean, False),
    }


class SubjectNameFlagsV3(AWSProperty):
    """
    `SubjectNameFlagsV3 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-subjectnameflagsv3.html>`__
    """

    props: PropsDictType = {
        "RequireCommonName": (boolean, False),
        "RequireDirectoryPath": (boolean, False),
        "RequireDnsAsCn": (boolean, False),
        "RequireEmail": (boolean, False),
        "SanRequireDirectoryGuid": (boolean, False),
        "SanRequireDns": (boolean, False),
        "SanRequireDomainDns": (boolean, False),
        "SanRequireEmail": (boolean, False),
        "SanRequireSpn": (boolean, False),
        "SanRequireUpn": (boolean, False),
    }


class TemplateV3(AWSProperty):
    """
    `TemplateV3 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-templatev3.html>`__
    """

    props: PropsDictType = {
        "CertificateValidity": (CertificateValidity, True),
        "EnrollmentFlags": (EnrollmentFlagsV3, True),
        "Extensions": (ExtensionsV3, True),
        "GeneralFlags": (GeneralFlagsV3, True),
        "HashAlgorithm": (str, True),
        "PrivateKeyAttributes": (PrivateKeyAttributesV3, True),
        "PrivateKeyFlags": (PrivateKeyFlagsV3, True),
        "SubjectNameFlags": (SubjectNameFlagsV3, True),
        "SupersededTemplates": ([str], False),
    }


class EnrollmentFlagsV4(AWSProperty):
    """
    `EnrollmentFlagsV4 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-enrollmentflagsv4.html>`__
    """

    props: PropsDictType = {
        "EnableKeyReuseOnNtTokenKeysetStorageFull": (boolean, False),
        "IncludeSymmetricAlgorithms": (boolean, False),
        "NoSecurityExtension": (boolean, False),
        "RemoveInvalidCertificateFromPersonalStore": (boolean, False),
        "UserInteractionRequired": (boolean, False),
    }


class ExtensionsV4(AWSProperty):
    """
    `ExtensionsV4 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-extensionsv4.html>`__
    """

    props: PropsDictType = {
        "ApplicationPolicies": (ApplicationPolicies, False),
        "KeyUsage": (KeyUsage, True),
    }


class GeneralFlagsV4(AWSProperty):
    """
    `GeneralFlagsV4 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-generalflagsv4.html>`__
    """

    props: PropsDictType = {
        "AutoEnrollment": (boolean, False),
        "MachineType": (boolean, False),
    }


class PrivateKeyAttributesV4(AWSProperty):
    """
    `PrivateKeyAttributesV4 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-privatekeyattributesv4.html>`__
    """

    props: PropsDictType = {
        "Algorithm": (str, False),
        "CryptoProviders": ([str], False),
        "KeySpec": (str, True),
        "KeyUsageProperty": (KeyUsageProperty, False),
        "MinimalKeyLength": (double, True),
    }


class PrivateKeyFlagsV4(AWSProperty):
    """
    `PrivateKeyFlagsV4 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-privatekeyflagsv4.html>`__
    """

    props: PropsDictType = {
        "ClientVersion": (str, True),
        "ExportableKey": (boolean, False),
        "RequireAlternateSignatureAlgorithm": (boolean, False),
        "RequireSameKeyRenewal": (boolean, False),
        "StrongKeyProtectionRequired": (boolean, False),
        "UseLegacyProvider": (boolean, False),
    }


class SubjectNameFlagsV4(AWSProperty):
    """
    `SubjectNameFlagsV4 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-subjectnameflagsv4.html>`__
    """

    props: PropsDictType = {
        "RequireCommonName": (boolean, False),
        "RequireDirectoryPath": (boolean, False),
        "RequireDnsAsCn": (boolean, False),
        "RequireEmail": (boolean, False),
        "SanRequireDirectoryGuid": (boolean, False),
        "SanRequireDns": (boolean, False),
        "SanRequireDomainDns": (boolean, False),
        "SanRequireEmail": (boolean, False),
        "SanRequireSpn": (boolean, False),
        "SanRequireUpn": (boolean, False),
    }


class TemplateV4(AWSProperty):
    """
    `TemplateV4 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-templatev4.html>`__
    """

    props: PropsDictType = {
        "CertificateValidity": (CertificateValidity, True),
        "EnrollmentFlags": (EnrollmentFlagsV4, True),
        "Extensions": (ExtensionsV4, True),
        "GeneralFlags": (GeneralFlagsV4, True),
        "HashAlgorithm": (str, False),
        "PrivateKeyAttributes": (PrivateKeyAttributesV4, True),
        "PrivateKeyFlags": (PrivateKeyFlagsV4, True),
        "SubjectNameFlags": (SubjectNameFlagsV4, True),
        "SupersededTemplates": ([str], False),
    }


class TemplateDefinition(AWSProperty):
    """
    `TemplateDefinition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-template-templatedefinition.html>`__
    """

    props: PropsDictType = {
        "TemplateV2": (TemplateV2, False),
        "TemplateV3": (TemplateV3, False),
        "TemplateV4": (TemplateV4, False),
    }


class Template(AWSObject):
    """
    `Template <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-pcaconnectorad-template.html>`__
    """

    resource_type = "AWS::PCAConnectorAD::Template"

    props: PropsDictType = {
        "ConnectorArn": (str, True),
        "Definition": (TemplateDefinition, True),
        "Name": (str, True),
        "ReenrollAllCertificateHolders": (boolean, False),
        "Tags": (dict, False),
    }


class AccessRights(AWSProperty):
    """
    `AccessRights <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-pcaconnectorad-templategroupaccesscontrolentry-accessrights.html>`__
    """

    props: PropsDictType = {
        "AutoEnroll": (str, False),
        "Enroll": (str, False),
    }


class TemplateGroupAccessControlEntry(AWSObject):
    """
    `TemplateGroupAccessControlEntry <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-pcaconnectorad-templategroupaccesscontrolentry.html>`__
    """

    resource_type = "AWS::PCAConnectorAD::TemplateGroupAccessControlEntry"

    props: PropsDictType = {
        "AccessRights": (AccessRights, True),
        "GroupDisplayName": (str, True),
        "GroupSecurityIdentifier": (str, False),
        "TemplateArn": (str, False),
    }



import unittest

from troposphere import Template, efs


class TestEfsTemplate(unittest.TestCase):
    def test_bucket_template(self):
        template = Template()
        title = "Efs"
        efs.FileSystem(title, template)
        self.assertIn(title, template.resources)


class TestEfs(unittest.TestCase):
    def test_validData(self):
        file_system = efs.FileSystem("Efs")
        file_system.to_dict()

    def test_validateThroughputMode(self):
        with self.assertRaises(ValueError):
            file_system = efs.FileSystem(
                "Efs", ThroughputMode="UndefinedThroughputMode"
            )
            file_system.to_dict()

        file_system = efs.FileSystem("Efs", ThroughputMode=efs.Bursting)

        result = file_system.to_dict()
        self.assertEqual(result["Type"], "AWS::EFS::FileSystem")

    def test_validateProvisionedThroughputInMibps(self):
        result = efs.FileSystem("Efs", ProvisionedThroughputInMibps=512.0).to_dict()
        self.assertEqual(result["Type"], "AWS::EFS::FileSystem")

        efs.FileSystem("Efs", ProvisionedThroughputInMibps=512).to_dict()
        efs.FileSystem("Efs", ProvisionedThroughputInMibps=512.0).to_dict()

        with self.assertRaises(TypeError):
            efs.FileSystem("Efs", ProvisionedThroughputInMibps="512").to_dict()
        with self.assertRaises(ValueError):
            efs.FileSystem("Efs", ProvisionedThroughputInMibps=-512.0).to_dict()
        with self.assertRaises(ValueError):
            efs.FileSystem("Efs", ProvisionedThroughputInMibps=-512).to_dict()

    def test_validateBackupPolicy(self):
        with self.assertRaises(ValueError):
            backup_policy = efs.BackupPolicy("backupPolicy", Status="NOTOK")
            backup_policy.to_dict()

        backup_policy = efs.BackupPolicy("backupPolicy", Status="ENABLED")

        result = backup_policy.to_dict()
        self.assertEqual(result["Status"], "ENABLED")


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import integer


class Repository(AWSProperty):
    """
    `Repository <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloud9-environmentec2-repository.html>`__
    """

    props: PropsDictType = {
        "PathComponent": (str, True),
        "RepositoryUrl": (str, True),
    }


class EnvironmentEC2(AWSObject):
    """
    `EnvironmentEC2 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloud9-environmentec2.html>`__
    """

    resource_type = "AWS::Cloud9::EnvironmentEC2"

    props: PropsDictType = {
        "AutomaticStopTimeMinutes": (integer, False),
        "ConnectionType": (str, False),
        "Description": (str, False),
        "ImageId": (str, True),
        "InstanceType": (str, True),
        "Name": (str, False),
        "OwnerArn": (str, False),
        "Repositories": ([Repository], False),
        "SubnetId": (str, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, double, integer


class InsightsConfiguration(AWSProperty):
    """
    `InsightsConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-xray-group-insightsconfiguration.html>`__
    """

    props: PropsDictType = {
        "InsightsEnabled": (boolean, False),
        "NotificationsEnabled": (boolean, False),
    }


class Group(AWSObject):
    """
    `Group <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-xray-group.html>`__
    """

    resource_type = "AWS::XRay::Group"

    props: PropsDictType = {
        "FilterExpression": (str, False),
        "GroupName": (str, True),
        "InsightsConfiguration": (InsightsConfiguration, False),
        "Tags": (Tags, False),
    }


class ResourcePolicy(AWSObject):
    """
    `ResourcePolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-xray-resourcepolicy.html>`__
    """

    resource_type = "AWS::XRay::ResourcePolicy"

    props: PropsDictType = {
        "BypassPolicyLockoutCheck": (boolean, False),
        "PolicyDocument": (str, True),
        "PolicyName": (str, True),
    }


class SamplingRuleProperty(AWSProperty):
    """
    `SamplingRuleProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-xray-samplingrule-samplingrule.html>`__
    """

    props: PropsDictType = {
        "Attributes": (dict, False),
        "FixedRate": (double, True),
        "HTTPMethod": (str, True),
        "Host": (str, True),
        "Priority": (integer, True),
        "ReservoirSize": (integer, True),
        "ResourceARN": (str, True),
        "RuleARN": (str, False),
        "RuleName": (str, False),
        "ServiceName": (str, True),
        "ServiceType": (str, True),
        "URLPath": (str, True),
        "Version": (integer, False),
    }


class SamplingRule(AWSObject):
    """
    `SamplingRule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-xray-samplingrule.html>`__
    """

    resource_type = "AWS::XRay::SamplingRule"

    props: PropsDictType = {
        "SamplingRule": (SamplingRuleProperty, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags


class DRTAccess(AWSObject):
    """
    `DRTAccess <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-shield-drtaccess.html>`__
    """

    resource_type = "AWS::Shield::DRTAccess"

    props: PropsDictType = {
        "LogBucketList": ([str], False),
        "RoleArn": (str, True),
    }


class EmergencyContact(AWSProperty):
    """
    `EmergencyContact <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-shield-proactiveengagement-emergencycontact.html>`__
    """

    props: PropsDictType = {
        "ContactNotes": (str, False),
        "EmailAddress": (str, True),
        "PhoneNumber": (str, False),
    }


class ProactiveEngagement(AWSObject):
    """
    `ProactiveEngagement <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-shield-proactiveengagement.html>`__
    """

    resource_type = "AWS::Shield::ProactiveEngagement"

    props: PropsDictType = {
        "EmergencyContactList": ([EmergencyContact], True),
        "ProactiveEngagementStatus": (str, True),
    }


class Action(AWSProperty):
    """
    `Action <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-shield-protection-action.html>`__
    """

    props: PropsDictType = {
        "Block": (dict, False),
        "Count": (dict, False),
    }


class ApplicationLayerAutomaticResponseConfiguration(AWSProperty):
    """
    `ApplicationLayerAutomaticResponseConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-shield-protection-applicationlayerautomaticresponseconfiguration.html>`__
    """

    props: PropsDictType = {
        "Action": (Action, True),
        "Status": (str, True),
    }


class Protection(AWSObject):
    """
    `Protection <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-shield-protection.html>`__
    """

    resource_type = "AWS::Shield::Protection"

    props: PropsDictType = {
        "ApplicationLayerAutomaticResponseConfiguration": (
            ApplicationLayerAutomaticResponseConfiguration,
            False,
        ),
        "HealthCheckArns": ([str], False),
        "Name": (str, True),
        "ResourceArn": (str, True),
        "Tags": (Tags, False),
    }


class ProtectionGroup(AWSObject):
    """
    `ProtectionGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-shield-protectiongroup.html>`__
    """

    resource_type = "AWS::Shield::ProtectionGroup"

    props: PropsDictType = {
        "Aggregation": (str, True),
        "Members": ([str], False),
        "Pattern": (str, True),
        "ProtectionGroupId": (str, True),
        "ResourceType": (str, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators.resourcegroups import resourcequery_type


class ConfigurationParameter(AWSProperty):
    """
    `ConfigurationParameter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resourcegroups-group-configurationparameter.html>`__
    """

    props: PropsDictType = {
        "Name": (str, False),
        "Values": ([str], False),
    }


class ConfigurationItem(AWSProperty):
    """
    `ConfigurationItem <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resourcegroups-group-configurationitem.html>`__
    """

    props: PropsDictType = {
        "Parameters": ([ConfigurationParameter], False),
        "Type": (str, False),
    }


class TagFilter(AWSProperty):
    """
    `TagFilter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resourcegroups-group-tagfilter.html>`__
    """

    props: PropsDictType = {
        "Key": (str, False),
        "Values": ([str], False),
    }


class Query(AWSProperty):
    """
    `Query <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resourcegroups-group-query.html>`__
    """

    props: PropsDictType = {
        "ResourceTypeFilters": ([str], False),
        "StackIdentifier": (str, False),
        "TagFilters": ([TagFilter], False),
    }


class ResourceQuery(AWSProperty):
    """
    `ResourceQuery <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resourcegroups-group-resourcequery.html>`__
    """

    props: PropsDictType = {
        "Query": (Query, False),
        "Type": (resourcequery_type, False),
    }


class Group(AWSObject):
    """
    `Group <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-resourcegroups-group.html>`__
    """

    resource_type = "AWS::ResourceGroups::Group"

    props: PropsDictType = {
        "Configuration": ([ConfigurationItem], False),
        "Description": (str, False),
        "Name": (str, True),
        "ResourceQuery": (ResourceQuery, False),
        "Resources": ([str], False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from . import integer_range


def validate_sortorder(x):
    """
    Property: Order.SortOrder
    """
    return integer_range(0, 1)(x)


def connection_type_validator(type):
    """
    Property: ConnectionInput.ConnectionType
    """
    valid_types = [
        "CUSTOM",
        "JDBC",
        "KAFKA",
        "MARKETPLACE",
        "MONGODB",
        "NETWORK",
        "SFTP",
    ]
    if type not in valid_types:
        raise ValueError("% is not a valid value for ConnectionType" % type)
    return type


def delete_behavior_validator(value):
    """
    Property: SchemaChangePolicy.DeleteBehavior
    """
    valid_values = [
        "LOG",
        "DELETE_FROM_DATABASE",
        "DEPRECATE_IN_DATABASE",
    ]
    if value not in valid_values:
        raise ValueError("% is not a valid value for DeleteBehavior" % value)
    return value


def update_behavior_validator(value):
    """
    Property: SchemaChangePolicy.UpdateBehavior
    """
    valid_values = [
        "LOG",
        "UPDATE_IN_DATABASE",
    ]
    if value not in valid_values:
        raise ValueError("% is not a valid value for UpdateBehavior" % value)
    return value


def table_type_validator(type):
    """
    Property: TableInput.TableType
    """
    valid_types = [
        "EXTERNAL_TABLE",
        "VIRTUAL_VIEW",
    ]
    if type not in valid_types:
        raise ValueError("% is not a valid value for TableType" % type)
    return type


def trigger_type_validator(type):
    """
    Property: Trigger.Type
    """
    valid_types = [
        "SCHEDULED",
        "CONDITIONAL",
        "ON_DEMAND",
        "EVENT",
    ]
    if type not in valid_types:
        raise ValueError("% is not a valid value for Type" % type)
    return type



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from ..compat import validate_policytype
from . import integer_range, tags_or_list


def policytypes(policy):
    """
    Property: Domain.AccessPolicies
    """
    return validate_policytype(policy)


def validate_automated_snapshot_start_hour(port):
    """
    Property: SnapshotOptions.AutomatedSnapshotStartHour
    """
    return integer_range(0, 23)(port)


def validate_tags_or_list(x):
    """
    Property: Domain.Tags
    """
    return tags_or_list(x)


def validate_volume_type(volume_type):
    """
    Validate VolumeType for ElasticsearchDomain
    Property: EBSOptions.VolumeType
    """

    VALID_VOLUME_TYPES = ("standard", "gp2", "gp3", "io1")

    if volume_type not in VALID_VOLUME_TYPES:
        raise ValueError(
            "Elasticsearch Domain VolumeType must be one of: %s"
            % ", ".join(VALID_VOLUME_TYPES)
        )
    return volume_type


def validate_tls_security_policy(tls_security_policy):
    """
    Validate TLS Security Policy for ElasticsearchDomain
    Property: DomainEndpointOptions.TLSSecurityPolicy
    """

    VALID_TLS_SECURITY_POLICIES = (
        "Policy-Min-TLS-1-0-2019-07",
        "Policy-Min-TLS-1-2-2019-07",
    )

    if tls_security_policy not in VALID_TLS_SECURITY_POLICIES:
        raise ValueError(
            "Minimum TLS Security Policy must be one of: %s"
            % ", ".join(VALID_TLS_SECURITY_POLICIES)
        )
    return tls_security_policy


def validate_ebs_options(self):
    """
    Class: EBSOptions
    """

    volume_type = self.properties.get("VolumeType")
    iops = self.properties.get("Iops")
    if volume_type == "io1" and not iops:
        raise ValueError("Must specify Iops if VolumeType is 'io1'.")
